@@ -1,32 +1,6 @@
-/*
-  Copyright 2008 Larry Gritz and the other authors and contributors.
-  All Rights Reserved.
-
-  Redistribution and use in source and binary forms, with or without
-  modification, are permitted provided that the following conditions are
-  met:
-  * Redistributions of source code must retain the above copyright
-    notice, this list of conditions and the following disclaimer.
-  * Redistributions in binary form must reproduce the above copyright
-    notice, this list of conditions and the following disclaimer in the
-    documentation and/or other materials provided with the distribution.
-  * Neither the name of the software's owners nor the names of its
-    contributors may be used to endorse or promote products derived from
-    this software without specific prior written permission.
-  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-
-  (This is the Modified BSD License)
-*/
+// Copyright 2008-present Contributors to the OpenImageIO project.
+// SPDX-License-Identifier: BSD-3-Clause
+// https://github.com/OpenImageIO/oiio/blob/master/LICENSE.md
 
 #include <cstdio>
 #include <cstdlib>
@@ -72,7 +46,7 @@ using namespace pvt;
 // type.
 template<class T>
 inline void
-get_default_quantize_(long long& quant_min, long long& quant_max)
+get_default_quantize_(long long& quant_min, long long& quant_max) noexcept
 {
     if (std::numeric_limits<T>::is_integer) {
         quant_min = (long long)std::numeric_limits<T>::min();
@@ -89,7 +63,7 @@ get_default_quantize_(long long& quant_min, long long& quant_max)
 // Rely on the template version to make life easy.
 void
 pvt::get_default_quantize(TypeDesc format, long long& quant_min,
-                          long long& quant_max)
+                          long long& quant_max) noexcept
 {
     switch (format.basetype) {
     case TypeDesc::UNKNOWN:
@@ -130,7 +104,7 @@ pvt::get_default_quantize(TypeDesc format, long long& quant_min,
 
 
 
-ImageSpec::ImageSpec(TypeDesc format)
+ImageSpec::ImageSpec(TypeDesc format) noexcept
     : x(0)
     , y(0)
     , z(0)
@@ -156,7 +130,7 @@ ImageSpec::ImageSpec(TypeDesc format)
 
 
 
-ImageSpec::ImageSpec(int xres, int yres, int nchans, TypeDesc format)
+ImageSpec::ImageSpec(int xres, int yres, int nchans, TypeDesc format) noexcept
     : x(0)
     , y(0)
     , z(0)
@@ -183,7 +157,7 @@ ImageSpec::ImageSpec(int xres, int yres, int nchans, TypeDesc format)
 
 
 
-ImageSpec::ImageSpec(const ROI& roi, TypeDesc format)
+ImageSpec::ImageSpec(const ROI& roi, TypeDesc format) noexcept
     : x(roi.xbegin)
     , y(roi.ybegin)
     , z(roi.zbegin)
@@ -211,7 +185,7 @@ ImageSpec::ImageSpec(const ROI& roi, TypeDesc format)
 
 
 void
-ImageSpec::set_format(TypeDesc fmt)
+ImageSpec::set_format(TypeDesc fmt) noexcept
 {
     format = fmt;
     channelformats.clear();
@@ -220,7 +194,7 @@ ImageSpec::set_format(TypeDesc fmt)
 
 
 void
-ImageSpec::default_channel_names()
+ImageSpec::default_channel_names() noexcept
 {
     channelnames.clear();
     channelnames.reserve(nchannels);
@@ -248,7 +222,7 @@ ImageSpec::default_channel_names()
 
 
 size_t
-ImageSpec::channel_bytes(int chan, bool native) const
+ImageSpec::channel_bytes(int chan, bool native) const noexcept
 {
     if (chan >= nchannels)
         return 0;
@@ -261,7 +235,7 @@ ImageSpec::channel_bytes(int chan, bool native) const
 
 
 size_t
-ImageSpec::pixel_bytes(bool native) const
+ImageSpec::pixel_bytes(bool native) const noexcept
 {
     if (nchannels < 0)
         return 0;
@@ -278,7 +252,7 @@ ImageSpec::pixel_bytes(bool native) const
 
 
 size_t
-ImageSpec::pixel_bytes(int chbegin, int chend, bool native) const
+ImageSpec::pixel_bytes(int chbegin, int chend, bool native) const noexcept
 {
     if (chbegin < 0)
         return 0;
@@ -296,7 +270,7 @@ ImageSpec::pixel_bytes(int chbegin, int chend, bool native) const
 
 
 imagesize_t
-ImageSpec::scanline_bytes(bool native) const
+ImageSpec::scanline_bytes(bool native) const noexcept
 {
     if (width < 0)
         return 0;
@@ -306,7 +280,7 @@ ImageSpec::scanline_bytes(bool native) const
 
 
 imagesize_t
-ImageSpec::tile_pixels() const
+ImageSpec::tile_pixels() const noexcept
 {
     if (tile_width <= 0 || tile_height <= 0 || tile_depth <= 0)
         return 0;
@@ -320,7 +294,7 @@ ImageSpec::tile_pixels() const
 
 
 imagesize_t
-ImageSpec::tile_bytes(bool native) const
+ImageSpec::tile_bytes(bool native) const noexcept
 {
     return clamped_mult64(tile_pixels(), (imagesize_t)pixel_bytes(native));
 }
@@ -328,7 +302,7 @@ ImageSpec::tile_bytes(bool native) const
 
 
 imagesize_t
-ImageSpec::image_pixels() const
+ImageSpec::image_pixels() const noexcept
 {
     if (width < 0 || height < 0 || depth < 0)
         return 0;
@@ -341,7 +315,7 @@ ImageSpec::image_pixels() const
 
 
 imagesize_t
-ImageSpec::image_bytes(bool native) const
+ImageSpec::image_bytes(bool native) const noexcept
 {
     return clamped_mult64(image_pixels(), (imagesize_t)pixel_bytes(native));
 }
@@ -470,7 +444,8 @@ ImageSpec::find_attribute(string_view name, ParamValue& tmpparam,
     GETINT(tile_depth);
     GETINT(alpha_channel);
     GETINT(z_channel);
-    // some special cases
+
+    // some special cases -- assemblies of multiple fields or attributes
     if (MATCH("geom", TypeString)) {
         ustring s = (depth <= 1 && full_depth <= 1)
                         ? ustring::sprintf("%dx%d%+d%+d", width, height, x, y)
@@ -489,6 +464,34 @@ ImageSpec::find_attribute(string_view name, ParamValue& tmpparam,
         tmpparam.init("full_geom", TypeString, 1, &s);
         return &tmpparam;
     }
+    static constexpr TypeDesc TypeInt_4(TypeDesc::INT, 4);
+    static constexpr TypeDesc TypeInt_6(TypeDesc::INT, 6);
+    if (MATCH("datawindow", TypeInt_4)) {
+        int val[] = { x, y, x + width - 1, y + height - 1 };
+        tmpparam.init(name, TypeInt_4, 1, &val);
+        return &tmpparam;
+    }
+    if (MATCH("datawindow", TypeInt_6)) {
+        int val[] = { x, y, z, x + width - 1, y + height - 1, z + depth - 1 };
+        tmpparam.init(name, TypeInt_6, 1, &val);
+        return &tmpparam;
+    }
+    if (MATCH("displaywindow", TypeInt_4)) {
+        int val[] = { full_x, full_y, full_x + full_width - 1,
+                      full_y + full_height - 1 };
+        tmpparam.init(name, TypeInt_4, 1, &val);
+        return &tmpparam;
+    }
+    if (MATCH("displaywindow", TypeInt_6)) {
+        int val[] = { full_x,
+                      full_y,
+                      full_z,
+                      full_x + full_width - 1,
+                      full_y + full_height - 1,
+                      full_z + full_depth - 1 };
+        tmpparam.init(name, TypeInt_6, 1, &val);
+        return &tmpparam;
+    }
 #undef GETINT
 #undef MATCH
     return NULL;
@@ -496,6 +499,31 @@ ImageSpec::find_attribute(string_view name, ParamValue& tmpparam,
 
 
 
+TypeDesc
+ImageSpec::getattributetype(string_view name, bool casesensitive) const
+{
+    ParamValue pv;
+    auto p = find_attribute(name, pv, TypeUnknown, casesensitive);
+    return p ? p->type() : TypeUnknown;
+}
+
+
+
+bool
+ImageSpec::getattribute(string_view name, TypeDesc type, void* value,
+                        bool casesensitive) const
+{
+    ParamValue pv;
+    auto p = find_attribute(name, pv, TypeUnknown, casesensitive);
+    if (p) {
+        return convert_type(p->type(), p->data(), type, value);
+    } else {
+        return false;
+    }
+}
+
+
+
 int
 ImageSpec::get_int_attribute(string_view name, int defaultval) const
 {
@@ -1088,27 +1116,27 @@ ImageSpec::from_xml(const char* xml)
     //int version = n.attribute ("version").as_int();
 
     // Fields for version == 10 (current)
-    x           = atoi(n.child_value("x"));
-    y           = atoi(n.child_value("y"));
-    z           = atoi(n.child_value("z"));
-    width       = atoi(n.child_value("width"));
-    height      = atoi(n.child_value("height"));
-    depth       = atoi(n.child_value("depth"));
-    full_x      = atoi(n.child_value("full_x"));
-    full_y      = atoi(n.child_value("full_y"));
-    full_z      = atoi(n.child_value("full_z"));
-    full_width  = atoi(n.child_value("full_width"));
-    full_height = atoi(n.child_value("full_height"));
-    full_depth  = atoi(n.child_value("full_depth"));
-    tile_width  = atoi(n.child_value("tile_width"));
-    tile_height = atoi(n.child_value("tile_height"));
-    tile_depth  = atoi(n.child_value("tile_depth"));
+    x           = Strutil::stoi(n.child_value("x"));
+    y           = Strutil::stoi(n.child_value("y"));
+    z           = Strutil::stoi(n.child_value("z"));
+    width       = Strutil::stoi(n.child_value("width"));
+    height      = Strutil::stoi(n.child_value("height"));
+    depth       = Strutil::stoi(n.child_value("depth"));
+    full_x      = Strutil::stoi(n.child_value("full_x"));
+    full_y      = Strutil::stoi(n.child_value("full_y"));
+    full_z      = Strutil::stoi(n.child_value("full_z"));
+    full_width  = Strutil::stoi(n.child_value("full_width"));
+    full_height = Strutil::stoi(n.child_value("full_height"));
+    full_depth  = Strutil::stoi(n.child_value("full_depth"));
+    tile_width  = Strutil::stoi(n.child_value("tile_width"));
+    tile_height = Strutil::stoi(n.child_value("tile_height"));
+    tile_depth  = Strutil::stoi(n.child_value("tile_depth"));
     format      = TypeDesc(n.child_value("format"));
-    nchannels   = atoi(n.child_value("nchannels"));
+    nchannels   = Strutil::stoi(n.child_value("nchannels"));
     get_channelnames(n, channelnames);
-    alpha_channel = atoi(n.child_value("alpha_channel"));
-    z_channel     = atoi(n.child_value("z_channel"));
-    deep          = atoi(n.child_value("deep"));
+    alpha_channel = Strutil::stoi(n.child_value("alpha_channel"));
+    z_channel     = Strutil::stoi(n.child_value("z_channel"));
+    deep          = Strutil::stoi(n.child_value("deep"));
 
     // FIXME: What about extra attributes?
 
@@ -1117,6 +1145,22 @@ ImageSpec::from_xml(const char* xml)
 
 
 
+std::pair<string_view, int>
+ImageSpec::decode_compression_metadata(string_view defaultcomp,
+                                       int defaultqual) const
+{
+    string_view comp   = get_string_attribute("Compression", defaultcomp);
+    int qual           = get_int_attribute("CompressionQuality", defaultqual);
+    auto comp_and_qual = Strutil::splitsv(comp, ":");
+    if (comp_and_qual.size() >= 1)
+        comp = comp_and_qual[0];
+    if (comp_and_qual.size() >= 2)
+        qual = Strutil::stoi(comp_and_qual[1]);
+    return { comp, qual };
+}
+
+
+
 bool
 pvt::check_texture_metadata_sanity(ImageSpec& spec)
 {
