@@ -1,36 +1,6 @@
-/*
-  Copyright 2008 Larry Gritz and the other authors and contributors.
-  All Rights Reserved.
-
-  Redistribution and use in source and binary forms, with or without
-  modification, are permitted provided that the following conditions are
-  met:
-  * Redistributions of source code must retain the above copyright
-    notice, this list of conditions and the following disclaimer.
-  * Redistributions in binary form must reproduce the above copyright
-    notice, this list of conditions and the following disclaimer in the
-    documentation and/or other materials provided with the distribution.
-  * Neither the name of the software's owners nor the names of its
-    contributors may be used to endorse or promote products derived from
-    this software without specific prior written permission.
-  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-
-  (This is the Modified BSD License)
-*/
-
-
-/// \file
-/// Implementation of ImageBuf class.
+// Copyright 2008-present Contributors to the OpenImageIO project.
+// SPDX-License-Identifier: BSD-3-Clause
+// https://github.com/OpenImageIO/oiio/blob/master/LICENSE.md
 
 
 #include <iostream>
@@ -282,12 +252,12 @@ private:
     size_t m_scanline_bytes;
     size_t m_plane_bytes;
     size_t m_channel_bytes;
-    ImageCache* m_imagecache;        ///< ImageCache to use
-    TypeDesc m_cachedpixeltype;      ///< Data type stored in the cache
-    DeepData m_deepdata;             ///< Deep data
-    size_t m_allocated_size;         ///< How much memory we've allocated
-    std::vector<char> m_blackpixel;  ///< Pixel-sized zero bytes
-    TypeDesc m_write_format;         /// Format to use for write()
+    ImageCache* m_imagecache;              ///< ImageCache to use
+    TypeDesc m_cachedpixeltype;            ///< Data type stored in the cache
+    DeepData m_deepdata;                   ///< Deep data
+    size_t m_allocated_size;               ///< How much memory we've allocated
+    std::vector<char> m_blackpixel;        ///< Pixel-sized zero bytes
+    std::vector<TypeDesc> m_write_format;  /// Pixel data format to use for write()
     int m_write_tile_width;
     int m_write_tile_height;
     int m_write_tile_depth;
@@ -300,12 +270,29 @@ private:
     // Private release of m_pixels.
     void free_pixels();
 
+    TypeDesc write_format(int channel = 0) const
+    {
+        if (size_t(channel) < m_write_format.size())
+            return m_write_format[channel];
+        if (m_write_format.size() == 1)
+            return m_write_format[0];
+        return m_nativespec.format;
+    }
+
     const ImageBufImpl operator=(const ImageBufImpl& src);  // unimplemented
     friend class ImageBuf;
 };
 
 
 
+void
+ImageBuf::impl_deleter(ImageBufImpl* todel)
+{
+    delete todel;
+}
+
+
+
 ImageBufImpl::ImageBufImpl(string_view filename, int subimage, int miplevel,
                            ImageCache* imagecache, const ImageSpec* spec,
                            void* buffer, const ImageSpec* config)
@@ -327,7 +314,6 @@ ImageBufImpl::ImageBufImpl(string_view filename, int subimage, int miplevel,
     , m_channel_bytes(0)
     , m_imagecache(imagecache)
     , m_allocated_size(0)
-    , m_write_format(TypeDesc::UNKNOWN)
     , m_write_tile_width(0)
     , m_write_tile_height(0)
     , m_write_tile_depth(1)
@@ -390,8 +376,7 @@ ImageBufImpl::ImageBufImpl(const ImageBufImpl& src)
     , m_cachedpixeltype(src.m_cachedpixeltype)
     , m_deepdata(src.m_deepdata)
     , m_allocated_size(0)
-    ,  // gets fixed up in the body vvv
-    m_blackpixel(src.m_blackpixel)
+    , m_blackpixel(src.m_blackpixel)  // gets fixed up in the body vvv
     , m_write_format(src.m_write_format)
     , m_write_tile_width(src.m_write_tile_width)
     , m_write_tile_height(src.m_write_tile_height)
@@ -431,7 +416,7 @@ ImageBufImpl::~ImageBufImpl()
 
 
 ImageBuf::ImageBuf()
-    : m_impl(new ImageBufImpl(std::string(), -1, -1, NULL))
+    : m_impl(new ImageBufImpl(std::string(), -1, -1, NULL), &impl_deleter)
 {
 }
 
@@ -440,51 +425,58 @@ ImageBuf::ImageBuf()
 ImageBuf::ImageBuf(string_view filename, int subimage, int miplevel,
                    ImageCache* imagecache, const ImageSpec* config)
     : m_impl(new ImageBufImpl(filename, subimage, miplevel, imagecache,
-                              NULL /*spec*/, NULL /*buffer*/, config))
+                              NULL /*spec*/, NULL /*buffer*/, config),
+             &impl_deleter)
 {
 }
 
 
 
 ImageBuf::ImageBuf(string_view filename, ImageCache* imagecache)
-    : m_impl(new ImageBufImpl(filename, 0, 0, imagecache))
+    : m_impl(new ImageBufImpl(filename, 0, 0, imagecache), &impl_deleter)
 {
 }
 
 
 
-ImageBuf::ImageBuf(const ImageSpec& spec)
-    : m_impl(new ImageBufImpl("", 0, 0, NULL, &spec))
+ImageBuf::ImageBuf(const ImageSpec& spec, InitializePixels zero)
+    : m_impl(new ImageBufImpl("", 0, 0, NULL, &spec), &impl_deleter)
 {
     m_impl->alloc(spec);
+    if (zero == InitializePixels::Yes && !deep())
+        ImageBufAlgo::zero(*this);
 }
 
 
 
-ImageBuf::ImageBuf(string_view filename, const ImageSpec& spec)
-    : m_impl(new ImageBufImpl(filename, 0, 0, NULL, &spec))
+ImageBuf::ImageBuf(string_view filename, const ImageSpec& spec,
+                   InitializePixels zero)
+    : m_impl(new ImageBufImpl(filename, 0, 0, NULL, &spec), &impl_deleter)
 {
     m_impl->alloc(spec);
+    if (zero == InitializePixels::Yes && !deep())
+        ImageBufAlgo::zero(*this);
 }
 
 
 
 ImageBuf::ImageBuf(string_view filename, const ImageSpec& spec, void* buffer)
-    : m_impl(new ImageBufImpl(filename, 0, 0, NULL, &spec, buffer))
+    : m_impl(new ImageBufImpl(filename, 0, 0, NULL, &spec, buffer),
+             &impl_deleter)
 {
 }
 
 
 
 ImageBuf::ImageBuf(const ImageSpec& spec, void* buffer)
-    : m_impl(new ImageBufImpl("", 0, 0, NULL, &spec, buffer))
+    : m_impl(new ImageBufImpl("", 0, 0, NULL, &spec, buffer), &impl_deleter)
 {
 }
 
 
 
 ImageBuf::ImageBuf(const ImageBuf& src)
-    : m_impl(new ImageBufImpl(*src.impl()))
+    : m_impl(new ImageBufImpl(*src.m_impl), &impl_deleter)
 {
 }
 
@@ -559,7 +551,7 @@ bool
 ImageBuf::has_error() const
 {
     spin_lock lock(err_mutex);
-    return !impl()->m_err.empty();
+    return !m_impl->m_err.empty();
 }
 
 
@@ -568,8 +560,8 @@ std::string
 ImageBuf::geterror(void) const
 {
     spin_lock lock(err_mutex);
-    std::string e = impl()->m_err;
-    impl()->m_err.clear();
+    std::string e = m_impl->m_err;
+    m_impl->m_err.clear();
     return e;
 }
 
@@ -591,7 +583,7 @@ ImageBufImpl::error(const std::string& message) const
 void
 ImageBuf::error(const std::string& message) const
 {
-    impl()->error(message);
+    m_impl->error(message);
 }
 
 
@@ -599,7 +591,7 @@ ImageBuf::error(const std::string& message) const
 ImageBuf::IBStorage
 ImageBuf::storage() const
 {
-    return impl()->storage();
+    return m_impl->storage();
 }
 
 
@@ -628,7 +620,7 @@ ImageBufImpl::clear()
     m_imagecache     = NULL;
     m_deepdata.free();
     m_blackpixel.clear();
-    m_write_format      = TypeDesc::UNKNOWN;
+    m_write_format.clear();
     m_write_tile_width  = 0;
     m_write_tile_height = 0;
     m_write_tile_depth  = 0;
@@ -640,7 +632,7 @@ ImageBufImpl::clear()
 void
 ImageBuf::clear()
 {
-    impl()->clear();
+    m_impl->clear();
 }
 
 
@@ -672,7 +664,7 @@ void
 ImageBuf::reset(string_view filename, int subimage, int miplevel,
                 ImageCache* imagecache, const ImageSpec* config)
 {
-    impl()->reset(filename, subimage, miplevel, imagecache, config);
+    m_impl->reset(filename, subimage, miplevel, imagecache, config);
 }
 
 
@@ -680,7 +672,7 @@ ImageBuf::reset(string_view filename, int subimage, int miplevel,
 void
 ImageBuf::reset(string_view filename, ImageCache* imagecache)
 {
-    impl()->reset(filename, 0, 0, imagecache, NULL);
+    m_impl->reset(filename, 0, 0, imagecache, NULL);
 }
 
 
@@ -701,17 +693,22 @@ ImageBufImpl::reset(string_view filename, const ImageSpec& spec,
 
 
 void
-ImageBuf::reset(string_view filename, const ImageSpec& spec)
+ImageBuf::reset(string_view filename, const ImageSpec& spec,
+                InitializePixels zero)
 {
-    impl()->reset(filename, spec);
+    m_impl->reset(filename, spec);
+    if (zero == InitializePixels::Yes && !deep())
+        ImageBufAlgo::zero(*this);
 }
 
 
 
 void
-ImageBuf::reset(const ImageSpec& spec)
+ImageBuf::reset(const ImageSpec& spec, InitializePixels zero)
 {
-    impl()->reset("", spec);
+    m_impl->reset("", spec);
+    if (zero == InitializePixels::Yes && !deep())
+        ImageBufAlgo::zero(*this);
 }
 
 
@@ -778,8 +775,13 @@ ImageBufImpl::init_spec(string_view filename, int subimage, int miplevel)
     m_nmiplevels   = 0;
     static ustring s_subimages("subimages"), s_miplevels("miplevels");
     static ustring s_fileformat("fileformat");
-    if (m_configspec)  // Pass configuration options to cache
-        m_imagecache->add_file(m_name, nullptr, m_configspec.get());
+    if (m_configspec) {  // Pass configuration options to cache
+        // Invalidate the file in the cache, and add with replacement
+        // because it might have a different config than last time.
+        m_imagecache->invalidate(m_name, true);
+        m_imagecache->add_file(m_name, nullptr, m_configspec.get(),
+                               /*replace=*/true);
+    }
     m_imagecache->get_image_info(m_name, subimage, miplevel, s_subimages,
                                  TypeInt, &m_nsubimages);
     m_imagecache->get_image_info(m_name, subimage, miplevel, s_miplevels,
@@ -836,7 +838,7 @@ ImageBufImpl::init_spec(string_view filename, int subimage, int miplevel)
 bool
 ImageBuf::init_spec(string_view filename, int subimage, int miplevel)
 {
-    return impl()->init_spec(filename, subimage, miplevel);
+    return m_impl->init_spec(filename, subimage, miplevel);
 }
 
 
@@ -869,12 +871,12 @@ ImageBufImpl::read(int subimage, int miplevel, int chbegin, int chend,
     if (m_spec.deep) {
         auto input = ImageInput::open(m_name.string(), m_configspec.get());
         if (!input) {
-            error("%s", OIIO::geterror());
+            errorf("%s", OIIO::geterror());
             return false;
         }
         input->threads(threads());  // Pass on our thread policy
         if (!input->read_native_deep_image(subimage, miplevel, m_deepdata)) {
-            error("%s", input->geterror());
+            errorf("%s", input->geterror());
             return false;
         }
         m_spec         = m_nativespec;  // Deep images always use native data
@@ -969,18 +971,22 @@ ImageBufImpl::read(int subimage, int miplevel, int chbegin, int chend,
                 ImageSpec newspec;
                 ok &= in->seek_subimage(subimage, miplevel, newspec);
             }
-            if (ok)
-                ok &= in->read_image(chbegin, chend, convert, m_localpixels);
+            if (ok) {
+                ok &= in->read_image(chbegin, chend, convert, m_localpixels,
+                                     AutoStride, AutoStride, AutoStride,
+                                     progress_callback, progress_callback_data);
+            }
+
             in->close();
             if (ok) {
                 m_pixels_valid = true;
             } else {
                 m_pixels_valid = false;
-                error("%s", in->geterror());
+                errorf("%s", in->geterror());
             }
         } else {
             m_pixels_valid = false;
-            error("%s", OIIO::geterror());
+            errorf("%s", OIIO::geterror());
         }
         return m_pixels_valid;
     }
@@ -995,7 +1001,7 @@ ImageBufImpl::read(int subimage, int miplevel, int chbegin, int chend,
         m_pixels_valid = true;
     } else {
         m_pixels_valid = false;
-        error("%s", m_imagecache->geterror());
+        errorf("%s", m_imagecache->geterror());
     }
 
     return m_pixels_valid;
@@ -1007,7 +1013,7 @@ bool
 ImageBuf::read(int subimage, int miplevel, bool force, TypeDesc convert,
                ProgressCallback progress_callback, void* progress_callback_data)
 {
-    return impl()->read(subimage, miplevel, 0, -1, force, convert,
+    return m_impl->read(subimage, miplevel, 0, -1, force, convert,
                         progress_callback, progress_callback_data);
 }
 
@@ -1018,25 +1024,36 @@ ImageBuf::read(int subimage, int miplevel, int chbegin, int chend, bool force,
                TypeDesc convert, ProgressCallback progress_callback,
                void* progress_callback_data)
 {
-    return impl()->read(subimage, miplevel, chbegin, chend, force, convert,
+    return m_impl->read(subimage, miplevel, chbegin, chend, force, convert,
                         progress_callback, progress_callback_data);
 }
 
 
 
+void
+ImageBuf::set_write_format(cspan<TypeDesc> format)
+{
+    m_impl->m_write_format.clear();
+    if (format.size() > 0)
+        m_impl->m_write_format.assign(format.data(),
+                                      format.data() + format.size());
+}
+
+
 void
 ImageBuf::set_write_format(TypeDesc format)
 {
-    impl()->m_write_format = format;
+    set_write_format(cspan<TypeDesc>(format));
 }
 
 
+
 void
 ImageBuf::set_write_tiles(int width, int height, int depth)
 {
-    impl()->m_write_tile_width  = width;
-    impl()->m_write_tile_height = height;
-    impl()->m_write_tile_depth  = std::max(1, depth);
+    m_impl->m_write_tile_width  = width;
+    m_impl->m_write_tile_height = height;
+    m_impl->m_write_tile_depth  = std::max(1, depth);
 }
 
 
@@ -1045,20 +1062,19 @@ bool
 ImageBuf::write(ImageOutput* out, ProgressCallback progress_callback,
                 void* progress_callback_data) const
 {
-    stride_t as              = AutoStride;
-    bool ok                  = true;
-    const ImageBufImpl* impl = this->impl();
-    ok &= impl->validate_pixels();
-    const ImageSpec& bufspec(impl->m_spec);
+    stride_t as = AutoStride;
+    bool ok     = true;
+    ok &= m_impl->validate_pixels();
+    const ImageSpec& bufspec(m_impl->m_spec);
     const ImageSpec& outspec(out->spec());
     TypeDesc bufformat = spec().format;
-    if (impl->m_localpixels) {
+    if (m_impl->m_localpixels) {
         // In-core pixel buffer for the whole image
-        ok = out->write_image(bufformat, impl->m_localpixels, as, as, as,
+        ok = out->write_image(bufformat, m_impl->m_localpixels, as, as, as,
                               progress_callback, progress_callback_data);
     } else if (deep()) {
         // Deep image record
-        ok = out->write_deep_image(impl->m_deepdata);
+        ok = out->write_deep_image(m_impl->m_deepdata);
     } else {
         // The image we want to write is backed by ImageCache -- we must be
         // immediately writing out a file from disk, possibly with file
@@ -1128,7 +1144,7 @@ ImageBuf::write(ImageOutput* out, ProgressCallback progress_callback,
         }
     }
     if (!ok)
-        error("%s", out->geterror());
+        errorf("%s", out->geterror());
     return ok;
 }
 
@@ -1142,10 +1158,10 @@ ImageBuf::write(string_view _filename, TypeDesc dtype, string_view _fileformat,
     string_view filename   = _filename.size() ? _filename : name();
     string_view fileformat = _fileformat.size() ? _fileformat : filename;
     if (filename.size() == 0) {
-        error("ImageBuf::write() called with no filename");
+        errorf("ImageBuf::write() called with no filename");
         return false;
     }
-    impl()->validate_pixels();
+    m_impl->validate_pixels();
 
     // Two complications related to our reliance on ImageCache, as we are
     // writing this image:
@@ -1156,7 +1172,7 @@ ImageBuf::write(string_view _filename, TypeDesc dtype, string_view _fileformat,
         m_impl->read(subimage(), miplevel(), 0, -1, true /*force*/,
                      spec().format, nullptr, nullptr);
         if (storage() != LOCALBUFFER) {
-            error("ImageBuf overwriting %s but could not force read", name());
+            errorf("ImageBuf overwriting %s but could not force read", name());
             return false;
         }
     }
@@ -1174,7 +1190,7 @@ ImageBuf::write(string_view _filename, TypeDesc dtype, string_view _fileformat,
 
     auto out = ImageOutput::create(fileformat.c_str(), "" /* searchpath */);
     if (!out) {
-        error("%s", geterror());
+        errorf("%s", geterror());
         return false;
     }
     out->threads(threads());  // Pass on our thread policy
@@ -1183,27 +1199,41 @@ ImageBuf::write(string_view _filename, TypeDesc dtype, string_view _fileformat,
     // user can override via ImageBuf::set_write_tiles(), or by using the
     // variety of IB::write() that takes the open ImageOutput* directly.
     ImageSpec newspec = spec();
-    if (out->supports("tiles") && impl()->m_write_tile_width > 0) {
-        newspec.tile_width  = impl()->m_write_tile_width;
-        newspec.tile_height = impl()->m_write_tile_height;
-        newspec.tile_depth  = std::max(1, impl()->m_write_tile_depth);
+    if (out->supports("tiles") && m_impl->m_write_tile_width > 0) {
+        newspec.tile_width  = m_impl->m_write_tile_width;
+        newspec.tile_height = m_impl->m_write_tile_height;
+        newspec.tile_depth  = std::max(1, m_impl->m_write_tile_depth);
     } else {
         newspec.tile_width  = 0;
         newspec.tile_height = 0;
         newspec.tile_depth  = 0;
     }
-    // Allow for format override via ImageBuf::set_write_format()
-    if (dtype == TypeUnknown)
-        dtype = impl()->m_write_format;
+
+    // Process pixel data type overrides
     if (dtype != TypeUnknown) {
+        // This call's dtype param, if set, overrides everything else
         newspec.set_format(dtype);
         newspec.channelformats.clear();
+    } else if (m_impl->m_write_format.size() != 0) {
+        // If set_write_format was called for the ImageBuf, it overrides
+        if (m_impl->m_write_format.size())
+            newspec.set_format(m_impl->write_format());
+        else
+            newspec.set_format(nativespec().format);
+        newspec.channelformats = m_impl->m_write_format;
+        newspec.channelformats.resize(newspec.nchannels, newspec.format);
+        for (auto& f : newspec.channelformats)
+            if (f == TypeUnknown)
+                f = newspec.format;
     } else {
+        // No override on the ImageBuf, nor on this call to write(), so
+        // we just use what is known from the imagespec.
         newspec.set_format(nativespec().format);
         newspec.channelformats = nativespec().channelformats;
     }
+
     if (!out->open(filename.c_str(), newspec)) {
-        error("%s", out->geterror());
+        errorf("%s", out->geterror());
         return false;
     }
     if (!write(out.get(), progress_callback, progress_callback_data))
@@ -1221,7 +1251,7 @@ ImageBuf::make_writeable(bool keep_cache_type)
 {
     if (storage() == IMAGECACHE) {
         return read(subimage(), miplevel(), 0, -1, true /*force*/,
-                    keep_cache_type ? impl()->m_cachedpixeltype : TypeDesc());
+                    keep_cache_type ? m_impl->m_cachedpixeltype : TypeDesc());
     }
     return true;
 }
@@ -1260,7 +1290,7 @@ ImageBufImpl::copy_metadata(const ImageBufImpl& src)
 void
 ImageBuf::copy_metadata(const ImageBuf& src)
 {
-    impl()->copy_metadata(*src.impl());
+    m_impl->copy_metadata(*src.m_impl);
 }
 
 
@@ -1268,7 +1298,7 @@ ImageBuf::copy_metadata(const ImageBuf& src)
 const ImageSpec&
 ImageBuf::spec() const
 {
-    return impl()->spec();
+    return m_impl->spec();
 }
 
 
@@ -1276,7 +1306,7 @@ ImageBuf::spec() const
 ImageSpec&
 ImageBuf::specmod()
 {
-    return impl()->specmod();
+    return m_impl->specmod();
 }
 
 
@@ -1284,7 +1314,7 @@ ImageBuf::specmod()
 const ImageSpec&
 ImageBuf::nativespec() const
 {
-    return impl()->nativespec();
+    return m_impl->nativespec();
 }
 
 
@@ -1292,52 +1322,52 @@ ImageBuf::nativespec() const
 string_view
 ImageBuf::name(void) const
 {
-    return impl()->m_name;
+    return m_impl->m_name;
 }
 
 
 string_view
 ImageBuf::file_format_name(void) const
 {
-    impl()->validate_spec();
-    return impl()->m_fileformat;
+    m_impl->validate_spec();
+    return m_impl->m_fileformat;
 }
 
 
 int
 ImageBuf::subimage() const
 {
-    return impl()->m_current_subimage;
+    return m_impl->m_current_subimage;
 }
 
 
 int
 ImageBuf::nsubimages() const
 {
-    impl()->validate_spec();
-    return impl()->m_nsubimages;
+    m_impl->validate_spec();
+    return m_impl->m_nsubimages;
 }
 
 
 int
 ImageBuf::miplevel() const
 {
-    return impl()->m_current_miplevel;
+    return m_impl->m_current_miplevel;
 }
 
 
 int
 ImageBuf::nmiplevels() const
 {
-    impl()->validate_spec();
-    return impl()->m_nmiplevels;
+    m_impl->validate_spec();
+    return m_impl->m_nmiplevels;
 }
 
 
 int
 ImageBuf::nchannels() const
 {
-    return impl()->spec().nchannels;
+    return m_impl->spec().nchannels;
 }
 
 
@@ -1345,8 +1375,8 @@ ImageBuf::nchannels() const
 int
 ImageBuf::orientation() const
 {
-    impl()->validate_spec();
-    return impl()->spec().get_int_attribute("Orientation", 1);
+    m_impl->validate_spec();
+    return m_impl->spec().get_int_attribute("Orientation", 1);
 }
 
 
@@ -1354,7 +1384,7 @@ ImageBuf::orientation() const
 void
 ImageBuf::set_orientation(int orient)
 {
-    impl()->specmod().attribute("Orientation", orient);
+    m_impl->specmod().attribute("Orientation", orient);
 }
 
 
@@ -1362,7 +1392,7 @@ ImageBuf::set_orientation(int orient)
 bool
 ImageBuf::pixels_valid(void) const
 {
-    return impl()->m_pixels_valid;
+    return m_impl->m_pixels_valid;
 }
 
 
@@ -1370,7 +1400,7 @@ ImageBuf::pixels_valid(void) const
 TypeDesc
 ImageBuf::pixeltype() const
 {
-    return impl()->pixeltype();
+    return m_impl->pixeltype();
 }
 
 
@@ -1378,8 +1408,8 @@ ImageBuf::pixeltype() const
 void*
 ImageBuf::localpixels()
 {
-    impl()->validate_pixels();
-    return impl()->m_localpixels;
+    m_impl->validate_pixels();
+    return m_impl->m_localpixels;
 }
 
 
@@ -1387,8 +1417,8 @@ ImageBuf::localpixels()
 const void*
 ImageBuf::localpixels() const
 {
-    impl()->validate_pixels();
-    return impl()->m_localpixels;
+    m_impl->validate_pixels();
+    return m_impl->m_localpixels;
 }
 
 
@@ -1420,7 +1450,7 @@ ImageBuf::z_stride() const
 bool
 ImageBuf::cachedpixels() const
 {
-    return impl()->cachedpixels();
+    return m_impl->cachedpixels();
 }
 
 
@@ -1428,7 +1458,7 @@ ImageBuf::cachedpixels() const
 ImageCache*
 ImageBuf::imagecache() const
 {
-    return impl()->m_imagecache;
+    return m_impl->m_imagecache;
 }
 
 
@@ -1443,21 +1473,21 @@ ImageBuf::deep() const
 DeepData*
 ImageBuf::deepdata()
 {
-    return impl()->deepdata();
+    return m_impl->deepdata();
 }
 
 
 const DeepData*
 ImageBuf::deepdata() const
 {
-    return impl()->deepdata();
+    return m_impl->deepdata();
 }
 
 
 bool
 ImageBuf::initialized() const
 {
-    return impl()->initialized();
+    return m_impl->initialized();
 }
 
 
@@ -1465,7 +1495,7 @@ ImageBuf::initialized() const
 void
 ImageBuf::threads(int n) const
 {
-    impl()->threads(n);
+    m_impl->threads(n);
 }
 
 
@@ -1473,7 +1503,7 @@ ImageBuf::threads(int n) const
 int
 ImageBuf::threads() const
 {
-    return impl()->threads();
+    return m_impl->threads();
 }
 
 
@@ -1564,7 +1594,7 @@ ImageBuf::copy_pixels(const ImageBuf& src)
 bool
 ImageBuf::copy(const ImageBuf& src, TypeDesc format)
 {
-    src.impl()->validate_pixels();
+    src.m_impl->validate_pixels();
     if (this == &src)  // self-assignment
         return true;
     if (src.storage() == UNINITIALIZED) {  // buf = uninitialized
@@ -1572,12 +1602,12 @@ ImageBuf::copy(const ImageBuf& src, TypeDesc format)
         return true;
     }
     if (src.deep()) {
-        impl()->reset(src.name(), src.spec(), &src.nativespec());
-        impl()->m_deepdata = src.impl()->m_deepdata;
+        m_impl->reset(src.name(), src.spec(), &src.nativespec());
+        m_impl->m_deepdata = src.m_impl->m_deepdata;
         return true;
     }
     if (format.basetype == TypeDesc::UNKNOWN || src.deep())
-        impl()->reset(src.name(), src.spec(), &src.nativespec());
+        m_impl->reset(src.name(), src.spec(), &src.nativespec());
     else {
         ImageSpec newspec(src.spec());
         newspec.set_format(format);
@@ -1664,7 +1694,7 @@ interppixel_(const ImageBuf& img, float x, float y, float* pixel,
              ImageBuf::WrapMode wrap)
 {
     int n             = img.spec().nchannels;
-    float* localpixel = ALLOCA(float, n * 4);
+    float* localpixel = OIIO_ALLOCA(float, n * 4);
     float* p[4]       = { localpixel, localpixel + n, localpixel + 2 * n,
                     localpixel + 3 * n };
     x -= 0.5f;
@@ -1707,7 +1737,7 @@ ImageBuf::interppixel(float x, float y, float* pixel, WrapMode wrap) const
 void
 ImageBuf::interppixel_NDC(float x, float y, float* pixel, WrapMode wrap) const
 {
-    const ImageSpec& spec(impl()->spec());
+    const ImageSpec& spec(m_impl->spec());
     interppixel(static_cast<float>(spec.full_x)
                     + x * static_cast<float>(spec.full_width),
                 static_cast<float>(spec.full_y)
@@ -1721,7 +1751,7 @@ void
 ImageBuf::interppixel_NDC_full(float x, float y, float* pixel,
                                WrapMode wrap) const
 {
-    const ImageSpec& spec(impl()->spec());
+    const ImageSpec& spec(m_impl->spec());
     interppixel(static_cast<float>(spec.full_x)
                     + x * static_cast<float>(spec.full_width),
                 static_cast<float>(spec.full_y)
@@ -1789,7 +1819,7 @@ void
 ImageBuf::interppixel_bicubic_NDC(float x, float y, float* pixel,
                                   WrapMode wrap) const
 {
-    const ImageSpec& spec(impl()->spec());
+    const ImageSpec& spec(m_impl->spec());
     interppixel_bicubic(static_cast<float>(spec.full_x)
                             + x * static_cast<float>(spec.full_width),
                         static_cast<float>(spec.full_y)
@@ -1938,7 +1968,7 @@ ImageBuf::set_pixels(ROI roi, TypeDesc format, const void* data,
                      stride_t xstride, stride_t ystride, stride_t zstride)
 {
     if (!initialized()) {
-        error("Cannot set_pixels() on an uninitialized ImageBuf");
+        errorf("Cannot set_pixels() on an uninitialized ImageBuf");
         return false;
     }
     bool ok;
@@ -1955,10 +1985,10 @@ ImageBuf::set_pixels(ROI roi, TypeDesc format, const void* data,
 int
 ImageBuf::deep_samples(int x, int y, int z) const
 {
-    impl()->validate_pixels();
+    m_impl->validate_pixels();
     if (!deep())
         return 0;
-    int p = impl()->pixelindex(x, y, z, true);
+    int p = m_impl->pixelindex(x, y, z, true);
     return p >= 0 ? deepdata()->samples(p) : 0;
 }
 
@@ -1967,11 +1997,11 @@ ImageBuf::deep_samples(int x, int y, int z) const
 const void*
 ImageBuf::deep_pixel_ptr(int x, int y, int z, int c, int s) const
 {
-    impl()->validate_pixels();
+    m_impl->validate_pixels();
     if (!deep())
         return NULL;
     const ImageSpec& m_spec(spec());
-    int p = impl()->pixelindex(x, y, z, true);
+    int p = m_impl->pixelindex(x, y, z, true);
     if (p < 0 || c < 0 || c >= m_spec.nchannels)
         return NULL;
     return (s < deepdata()->samples(p)) ? deepdata()->data_ptr(p, c, s) : NULL;
@@ -1982,11 +2012,11 @@ ImageBuf::deep_pixel_ptr(int x, int y, int z, int c, int s) const
 float
 ImageBuf::deep_value(int x, int y, int z, int c, int s) const
 {
-    impl()->validate_pixels();
+    m_impl->validate_pixels();
     if (!deep())
         return 0.0f;
-    int p = impl()->pixelindex(x, y, z);
-    return impl()->m_deepdata.deep_value(p, c, s);
+    int p = m_impl->pixelindex(x, y, z);
+    return m_impl->m_deepdata.deep_value(p, c, s);
 }
 
 
@@ -1994,11 +2024,11 @@ ImageBuf::deep_value(int x, int y, int z, int c, int s) const
 uint32_t
 ImageBuf::deep_value_uint(int x, int y, int z, int c, int s) const
 {
-    impl()->validate_pixels();
+    m_impl->validate_pixels();
     if (!deep())
         return 0;
-    int p = impl()->pixelindex(x, y, z);
-    return impl()->m_deepdata.deep_value_uint(p, c, s);
+    int p = m_impl->pixelindex(x, y, z);
+    return m_impl->m_deepdata.deep_value_uint(p, c, s);
 }
 
 
@@ -2008,8 +2038,8 @@ ImageBuf::set_deep_samples(int x, int y, int z, int samps)
 {
     if (!deep())
         return;
-    int p = impl()->pixelindex(x, y, z);
-    impl()->m_deepdata.set_samples(p, samps);
+    int p = m_impl->pixelindex(x, y, z);
+    m_impl->m_deepdata.set_samples(p, samps);
 }
 
 
@@ -2019,8 +2049,8 @@ ImageBuf::deep_insert_samples(int x, int y, int z, int samplepos, int nsamples)
 {
     if (!deep())
         return;
-    int p = impl()->pixelindex(x, y, z);
-    impl()->m_deepdata.insert_samples(p, samplepos, nsamples);
+    int p = m_impl->pixelindex(x, y, z);
+    m_impl->m_deepdata.insert_samples(p, samplepos, nsamples);
 }
 
 
@@ -2030,8 +2060,8 @@ ImageBuf::deep_erase_samples(int x, int y, int z, int samplepos, int nsamples)
 {
     if (!deep())
         return;
-    int p = impl()->pixelindex(x, y, z);
-    impl()->m_deepdata.erase_samples(p, samplepos, nsamples);
+    int p = m_impl->pixelindex(x, y, z);
+    m_impl->m_deepdata.erase_samples(p, samplepos, nsamples);
 }
 
 
@@ -2039,11 +2069,11 @@ ImageBuf::deep_erase_samples(int x, int y, int z, int samplepos, int nsamples)
 void
 ImageBuf::set_deep_value(int x, int y, int z, int c, int s, float value)
 {
-    impl()->validate_pixels();
+    m_impl->validate_pixels();
     if (!deep())
         return;
-    int p = impl()->pixelindex(x, y, z);
-    return impl()->m_deepdata.set_deep_value(p, c, s, value);
+    int p = m_impl->pixelindex(x, y, z);
+    return m_impl->m_deepdata.set_deep_value(p, c, s, value);
 }
 
 
@@ -2051,11 +2081,26 @@ ImageBuf::set_deep_value(int x, int y, int z, int c, int s, float value)
 void
 ImageBuf::set_deep_value(int x, int y, int z, int c, int s, uint32_t value)
 {
-    impl()->validate_pixels();
+    m_impl->validate_pixels();
     if (!deep())
         return;
-    int p = impl()->pixelindex(x, y, z);
-    return impl()->m_deepdata.set_deep_value(p, c, s, value);
+    int p = m_impl->pixelindex(x, y, z);
+    return m_impl->m_deepdata.set_deep_value(p, c, s, value);
+}
+
+
+
+bool
+ImageBuf::copy_deep_pixel(int x, int y, int z, const ImageBuf& src, int srcx,
+                          int srcy, int srcz)
+{
+    m_impl->validate_pixels();
+    src.m_impl->validate_pixels();
+    if (!deep() || !src.deep())
+        return false;
+    int p    = pixelindex(x, y, z);
+    int srcp = src.pixelindex(srcx, srcy, srcz);
+    return m_impl->m_deepdata.copy_deep_pixel(p, *src.deepdata(), srcp);
 }
 
 
@@ -2158,8 +2203,7 @@ ImageBuf::zmax() const
 int
 ImageBuf::oriented_width() const
 {
-    const ImageBufImpl* impl(this->impl());
-    const ImageSpec& spec(impl->spec());
+    const ImageSpec& spec(m_impl->spec());
     return orientation() <= 4 ? spec.width : spec.height;
 }
 
@@ -2168,8 +2212,7 @@ ImageBuf::oriented_width() const
 int
 ImageBuf::oriented_height() const
 {
-    const ImageBufImpl* impl(this->impl());
-    const ImageSpec& spec(impl->spec());
+    const ImageSpec& spec(m_impl->spec());
     return orientation() <= 4 ? spec.height : spec.width;
 }
 
@@ -2178,8 +2221,7 @@ ImageBuf::oriented_height() const
 int
 ImageBuf::oriented_x() const
 {
-    const ImageBufImpl* impl(this->impl());
-    const ImageSpec& spec(impl->spec());
+    const ImageSpec& spec(m_impl->spec());
     return orientation() <= 4 ? spec.x : spec.y;
 }
 
@@ -2188,8 +2230,7 @@ ImageBuf::oriented_x() const
 int
 ImageBuf::oriented_y() const
 {
-    const ImageBufImpl* impl(this->impl());
-    const ImageSpec& spec(impl->spec());
+    const ImageSpec& spec(m_impl->spec());
     return orientation() <= 4 ? spec.y : spec.x;
 }
 
@@ -2198,8 +2239,7 @@ ImageBuf::oriented_y() const
 int
 ImageBuf::oriented_full_width() const
 {
-    const ImageBufImpl* impl(this->impl());
-    const ImageSpec& spec(impl->spec());
+    const ImageSpec& spec(m_impl->spec());
     return orientation() <= 4 ? spec.full_width : spec.full_height;
 }
 
@@ -2208,8 +2248,7 @@ ImageBuf::oriented_full_width() const
 int
 ImageBuf::oriented_full_height() const
 {
-    const ImageBufImpl* impl(this->impl());
-    const ImageSpec& spec(impl->spec());
+    const ImageSpec& spec(m_impl->spec());
     return orientation() <= 4 ? spec.full_height : spec.full_width;
 }
 
@@ -2218,8 +2257,7 @@ ImageBuf::oriented_full_height() const
 int
 ImageBuf::oriented_full_x() const
 {
-    const ImageBufImpl* impl(this->impl());
-    const ImageSpec& spec(impl->spec());
+    const ImageSpec& spec(m_impl->spec());
     return orientation() <= 4 ? spec.full_x : spec.full_y;
 }
 
@@ -2228,8 +2266,7 @@ ImageBuf::oriented_full_x() const
 int
 ImageBuf::oriented_full_y() const
 {
-    const ImageBufImpl* impl(this->impl());
-    const ImageSpec& spec(impl->spec());
+    const ImageSpec& spec(m_impl->spec());
     return orientation() <= 4 ? spec.full_y : spec.full_x;
 }
 
@@ -2238,7 +2275,7 @@ ImageBuf::oriented_full_y() const
 void
 ImageBuf::set_origin(int x, int y, int z)
 {
-    ImageSpec& spec(impl()->specmod());
+    ImageSpec& spec(m_impl->specmod());
     spec.x = x;
     spec.y = y;
     spec.z = z;
@@ -2250,7 +2287,7 @@ void
 ImageBuf::set_full(int xbegin, int xend, int ybegin, int yend, int zbegin,
                    int zend)
 {
-    ImageSpec& m_spec(impl()->specmod());
+    ImageSpec& m_spec(m_impl->specmod());
     m_spec.full_x      = xbegin;
     m_spec.full_y      = ybegin;
     m_spec.full_z      = zbegin;
@@ -2331,7 +2368,7 @@ ImageBufImpl::pixeladdr(int x, int y, int z, int ch)
 const void*
 ImageBuf::pixeladdr(int x, int y, int z, int ch) const
 {
-    return impl()->pixeladdr(x, y, z, ch);
+    return m_impl->pixeladdr(x, y, z, ch);
 }
 
 
@@ -2339,7 +2376,7 @@ ImageBuf::pixeladdr(int x, int y, int z, int ch) const
 void*
 ImageBuf::pixeladdr(int x, int y, int z, int ch)
 {
-    return impl()->pixeladdr(x, y, z, ch);
+    return m_impl->pixeladdr(x, y, z, ch);
 }
 
 
@@ -2347,7 +2384,7 @@ ImageBuf::pixeladdr(int x, int y, int z, int ch)
 int
 ImageBuf::pixelindex(int x, int y, int z, bool check_range) const
 {
-    return impl()->pixelindex(x, y, z, check_range);
+    return m_impl->pixelindex(x, y, z, check_range);
 }
 
 
@@ -2355,7 +2392,7 @@ ImageBuf::pixelindex(int x, int y, int z, bool check_range) const
 const void*
 ImageBuf::blackpixel() const
 {
-    return impl()->blackpixel();
+    return m_impl->blackpixel();
 }
 
 
@@ -2467,7 +2504,7 @@ ImageBufImpl::retile(int x, int y, int z, ImageCache::Tile*& tile,
         if (!tile) {
             // Even though tile is NULL, ensure valid black pixel data
             std::string e = m_imagecache->geterror();
-            error("%s", e.size() ? e : "unspecified ImageCache error");
+            errorf("%s", e.size() ? e : "unspecified ImageCache error");
             return &m_blackpixel[0];
         }
     }
@@ -2491,7 +2528,7 @@ ImageBuf::retile(int x, int y, int z, ImageCache::Tile*& tile, int& tilexbegin,
                  int& tileybegin, int& tilezbegin, int& tilexend, bool exists,
                  WrapMode wrap) const
 {
-    return impl()->retile(x, y, z, tile, tilexbegin, tileybegin, tilezbegin,
+    return m_impl->retile(x, y, z, tile, tilexbegin, tileybegin, tilezbegin,
                           tilexend, exists, wrap);
 }
 
