@@ -1,32 +1,6 @@
-/*
-  Copyright 2012 Larry Gritz and the other authors and contributors.
-  All Rights Reserved.
-
-  Redistribution and use in source and binary forms, with or without
-  modification, are permitted provided that the following conditions are
-  met:
-  * Redistributions of source code must retain the above copyright
-  notice, this list of conditions and the following disclaimer.
-  * Redistributions in binary form must reproduce the above copyright
-  notice, this list of conditions and the following disclaimer in the
-  documentation and/or other materials provided with the distribution.
-  * Neither the name of the software's owners nor the names of its
-  contributors may be used to endorse or promote products derived from
-  this software without specific prior written permission.
-  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-
-  (This is the Modified BSD License)
-*/
+// Copyright 2008-present Contributors to the OpenImageIO project.
+// SPDX-License-Identifier: BSD-3-Clause
+// https://github.com/OpenImageIO/oiio/blob/master/LICENSE.md
 
 
 #include <OpenImageIO/benchmark.h>
@@ -340,7 +314,7 @@ test_read_channel_subset()
     std::cout << "\nTesting reading a channel subset\n";
 
     // FIrst, write a test image with 6 channels
-    static float color6[] = { .6, .5, .4, .3, .2, .1 };
+    static float color6[] = { 0.6f, 0.5f, 0.4f, 0.3f, 0.2f, 0.1f };
     ImageBuf A(ImageSpec(2, 2, 6, TypeDesc::FLOAT));
     ImageBufAlgo::fill(A, color6);
     A.write("sixchans.tif");
@@ -368,101 +342,6 @@ test_read_channel_subset()
 
 
 
-void
-test_write_png_to_memory()
-{
-    std::cout << "\nTesting writing a png file to a memory buffer\n";
-
-    // Make a 2x2 red image
-    ImageSpec spec(2, 2, 3, TypeUInt8);
-    ImageBuf buf(spec);
-    float red[3] = { 1, 0, 0 };
-    ImageBufAlgo::fill(buf, red);
-
-    // Use an IOVecOutput proxy to make the bytes go to file_buffer instead
-    // of to disk.
-    std::vector<unsigned char> file_buffer;
-    Filesystem::IOVecOutput memout(file_buffer);
-    void* ptr = &memout;
-    buf.specmod().attribute("oiio:ioproxy", TypeDesc::PTR, &ptr);
-
-    // Write the image. The proxy should do its magic.
-    buf.write("test.png");
-
-    // Check the results against the output of `hexdump test.png` when we
-    // saved the same image to disk. It should be byte-by-byte identical.
-    static const unsigned char reference[] = {
-        0x89, 0x50, 0x4e, 0x47, 0x0d, 0x0a, 0x1a, 0x0a, 0x00, 0x00, 0x00, 0x0d,
-        0x49, 0x48, 0x44, 0x52, 0x00, 0x00, 0x00, 0x02, 0x00, 0x00, 0x00, 0x02,
-        0x08, 0x02, 0x00, 0x00, 0x00, 0xfd, 0xd4, 0x9a, 0x73, 0x00, 0x00, 0x00,
-        0x09, 0x6f, 0x46, 0x46, 0x73, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
-        0x00, 0x00, 0xda, 0x2a, 0xb6, 0xce, 0x00, 0x00, 0x00, 0x10, 0x49, 0x44,
-        0x41, 0x54, 0x08, 0x99, 0x63, 0xfc, 0xcf, 0x00, 0x02, 0x4c, 0x60, 0x92,
-        0x01, 0x00, 0x0d, 0x1d, 0x01, 0x03, 0x6e, 0x28, 0x7c, 0x6d, 0x00, 0x00,
-        0x00, 0x00, 0x49, 0x45, 0x4e, 0x44, 0xae, 0x42, 0x60, 0x82
-    };
-    OIIO_CHECK_EQUAL(file_buffer.size(), 94ul);
-    OIIO_CHECK_ASSERT(
-        !memcmp(file_buffer.data(), reference, file_buffer.size()));
-    OIIO_CHECK_ASSERT(
-        !memcmp(file_buffer.data(), reference, file_buffer.size()));
-    // for (size_t i = 0; i < file_buffer_size; ++i)
-    //     Strutil::printf ("%c%02x", i%16 ? ' ' : '\n', file_buffer[i]);
-    // Strutil::printf ("\n");
-}
-
-
-
-void
-test_write_exr_to_memory()
-{
-    std::cout << "\nTesting writing an exr to a memory buffer\n";
-
-    // Make a 2x2 red image
-    ImageSpec spec(2, 2, 3, TypeHalf);
-    ImageBuf buf(spec);
-    float red[3] = { 1, 0, 0 };
-    ImageBufAlgo::fill(buf, red);
-
-    // Use an IOVecOutput proxy to make the bytes go to file_buffer instead
-    // of to disk.
-    std::vector<unsigned char> file_buffer;
-    Filesystem::IOVecOutput memout(file_buffer);
-    void* ptr = &memout;
-    buf.specmod().attribute("oiio:ioproxy", TypeDesc::PTR, &ptr);
-    // N.B. comment out the above line to write to disk
-
-    // Write the image. The proxy should do its magic.
-    buf.write("test.exr");
-
-    // Check the results against the known size of the disk output.
-    OIIO_CHECK_EQUAL(file_buffer.size(), 383ul);
-
-    // Now read it back again:
-    // * Set up an IOMemReader that reads from the buffer we have
-    Filesystem::IOMemReader memreader(file_buffer);
-    // * Make an config that specifies the memreader as IOProxy.
-    ImageSpec configspec;
-    ptr = &memreader;
-    configspec.attribute("oiio:ioproxy", TypeDesc::PTR, &ptr);
-    // * Make an ImageBuf that uses that config.
-    ImageBuf readbuf("test.exr", 0, 0, nullptr, &configspec);
-    // * The image we read should be the same as the one we started with.
-    OIIO_CHECK_EQUAL(readbuf.spec().height * readbuf.spec().width, 4);
-    for (int y = 0; y < readbuf.spec().height; ++y) {
-        for (int x = 0; x < readbuf.spec().width; ++x) {
-            float rpixel[3], bpixel[3];
-            readbuf.getpixel(x, y, rpixel);
-            buf.getpixel(x, y, bpixel);
-            OIIO_CHECK_EQUAL(rpixel[0], bpixel[0]);
-            OIIO_CHECK_EQUAL(rpixel[1], bpixel[1]);
-            OIIO_CHECK_EQUAL(rpixel[2], bpixel[2]);
-        }
-    }
-}
-
-
-
 void
 test_roi()
 {
@@ -527,9 +406,6 @@ main(int argc, char** argv)
     test_set_get_pixels();
     time_get_pixels();
 
-    test_write_png_to_memory();
-    test_write_exr_to_memory();
-
     Filesystem::remove("A_imagebuf_test.tif");
     return unit_test_failures;
 }
