@@ -1,32 +1,6 @@
-/*
-  Copyright 2008 Larry Gritz and the other authors and contributors.
-  All Rights Reserved.
-
-  Redistribution and use in source and binary forms, with or without
-  modification, are permitted provided that the following conditions are
-  met:
-  * Redistributions of source code must retain the above copyright
-    notice, this list of conditions and the following disclaimer.
-  * Redistributions in binary form must reproduce the above copyright
-    notice, this list of conditions and the following disclaimer in the
-    documentation and/or other materials provided with the distribution.
-  * Neither the name of the software's owners nor the names of its
-    contributors may be used to endorse or promote products derived from
-    this software without specific prior written permission.
-  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-
-  (This is the Modified BSD License)
-*/
+// Copyright 2008-present Contributors to the OpenImageIO project.
+// SPDX-License-Identifier: BSD-3-Clause
+// https://github.com/OpenImageIO/oiio/blob/master/LICENSE.md
 
 #include <cmath>
 #include <memory>
@@ -93,8 +67,7 @@ ImageBufAlgo::IBAprep(ROI& roi, ImageBuf* dst, const ImageBuf* A,
     ASSERT(dst);
     if ((A && !A->initialized()) || (B && !B->initialized())
         || (C && !C->initialized())) {
-        if (dst)
-            dst->error("Uninitialized input image");
+        dst->errorf("Uninitialized input image");
         return false;
     }
 
@@ -102,7 +75,7 @@ ImageBufAlgo::IBAprep(ROI& roi, ImageBuf* dst, const ImageBuf* A,
     if (dst || A || B || C) {
         minchans = 10000;
         maxchans = 1;
-        if (dst && dst->initialized()) {
+        if (dst->initialized()) {
             minchans = std::min(minchans, dst->spec().nchannels);
             maxchans = std::max(maxchans, dst->spec().nchannels);
         }
@@ -141,10 +114,23 @@ ImageBufAlgo::IBAprep(ROI& roi, ImageBuf* dst, const ImageBuf* A,
         // to fully read it into allocated memory so that we're able
         // to write to it subsequently.
         dst->make_writeable(true);
+
+        // Merge source metadata into destination if requested.
+        if (prepflags & IBAprep_MERGE_METADATA) {
+            if (A && A->initialized())
+                dst->specmod().extra_attribs.merge(A->spec().extra_attribs);
+            if (B && B->initialized())
+                dst->specmod().extra_attribs.merge(B->spec().extra_attribs);
+            if (C && C->initialized())
+                dst->specmod().extra_attribs.merge(C->spec().extra_attribs);
+        }
     } else {
         // Not an initialized destination image!
-        ASSERT((A || roi.defined())
-               && "ImageBufAlgo without any guess about region of interest");
+        if (!A && !roi.defined()) {
+            dst->errorf(
+                "ImageBufAlgo without any guess about region of interest");
+            return false;
+        }
         ROI full_roi;
         if (!roi.defined()) {
             // No ROI -- make it the union of the pixel regions of the inputs
@@ -242,6 +228,16 @@ ImageBufAlgo::IBAprep(ROI& roi, ImageBuf* dst, const ImageBuf* A,
         else
             set_roi_full(spec, roi);
 
+        // Merge source metadata into destination if requested.
+        if (prepflags & IBAprep_MERGE_METADATA) {
+            if (A && A->initialized())
+                spec.extra_attribs.merge(A->spec().extra_attribs);
+            if (B && B->initialized())
+                spec.extra_attribs.merge(B->spec().extra_attribs);
+            if (C && C->initialized())
+                spec.extra_attribs.merge(C->spec().extra_attribs);
+        }
+
         if (prepflags & IBAprep_NO_COPY_METADATA)
             spec.extra_attribs.clear();
         else if (!(prepflags & IBAprep_COPY_ALL_METADATA)) {
@@ -282,7 +278,7 @@ ImageBufAlgo::IBAprep(ROI& roi, ImageBuf* dst, const ImageBuf* A,
         if (dst->spec().alpha_channel < 0 || (A && A->spec().alpha_channel < 0)
             || (B && B->spec().alpha_channel < 0)
             || (C && C->spec().alpha_channel < 0)) {
-            dst->error("images must have alpha channels");
+            dst->errorf("images must have alpha channels");
             return false;
         }
     }
@@ -290,7 +286,7 @@ ImageBufAlgo::IBAprep(ROI& roi, ImageBuf* dst, const ImageBuf* A,
         if (dst->spec().z_channel < 0 || (A && A->spec().z_channel < 0)
             || (B && B->spec().z_channel < 0)
             || (C && C->spec().z_channel < 0)) {
-            dst->error("images must have depth channels");
+            dst->errorf("images must have depth channels");
             return false;
         }
     }
@@ -299,7 +295,7 @@ ImageBufAlgo::IBAprep(ROI& roi, ImageBuf* dst, const ImageBuf* A,
         int n = dst->spec().nchannels;
         if ((A && A->spec().nchannels != n) || (B && B->spec().nchannels != n)
             || (C && C->spec().nchannels != n)) {
-            dst->error("images must have the same number of channels");
+            dst->errorf("images must have the same number of channels");
             return false;
         }
     }
@@ -310,7 +306,8 @@ ImageBufAlgo::IBAprep(ROI& roi, ImageBuf* dst, const ImageBuf* A,
             if ((A && A->spec().channel_name(c) != name)
                 || (B && B->spec().channel_name(c) != name)
                 || (C && C->spec().channel_name(c) != name)) {
-                dst->error("images must have the same channel names and order");
+                dst->errorf(
+                    "images must have the same channel names and order");
                 return false;
             }
         }
@@ -318,23 +315,23 @@ ImageBufAlgo::IBAprep(ROI& roi, ImageBuf* dst, const ImageBuf* A,
     if (prepflags & IBAprep_NO_SUPPORT_VOLUME) {
         if (dst->spec().depth > 1 || (A && A->spec().depth > 1)
             || (B && B->spec().depth > 1) || (C && C->spec().depth > 1)) {
-            dst->error("volumes not supported");
+            dst->errorf("volumes not supported");
             return false;
         }
     }
-    if ((dst && dst->deep()) || (A && A->deep()) || (B && B->deep())
+    if (dst->deep() || (A && A->deep()) || (B && B->deep())
         || (C && C->deep())) {
         // At least one image is deep
         if (!(prepflags & IBAprep_SUPPORT_DEEP)) {
             // Error if the operation doesn't support deep images
-            dst->error("deep images not supported");
+            dst->errorf("deep images not supported");
             return false;
         }
         if (!(prepflags & IBAprep_DEEP_MIXED)) {
             // Error if not all images are deep
-            if ((dst && !dst->deep()) || (A && !A->deep()) || (B && !B->deep())
+            if (!dst->deep() || (A && !A->deep()) || (B && !B->deep())
                 || (C && !C->deep())) {
-                dst->error("mixed deep & flat images not supported");
+                dst->errorf("mixed deep & flat images not supported");
                 return false;
             }
         }
@@ -401,7 +398,7 @@ convolve_(ImageBuf& dst, const ImageBuf& src, const ImageBuf& kernel,
                 scale += k[0];
             scale = 1.0f / scale;
         }
-        float* sum = ALLOCA(float, roi.chend);
+        float* sum = OIIO_ALLOCA(float, roi.chend);
 
         ImageBuf::Iterator<DSTTYPE> d(dst, roi);
         ImageBuf::ConstIterator<SRCTYPE> s(src, roi, ImageBuf::WrapClamp);
@@ -457,7 +454,7 @@ ImageBufAlgo::convolve(const ImageBuf& src, const ImageBuf& kernel,
     ImageBuf result;
     bool ok = convolve(result, src, kernel, normalize, roi, nthreads);
     if (!ok && !result.has_error())
-        result.error("ImageBufAlgo::convolve() error");
+        result.errorf("ImageBufAlgo::convolve() error");
     return result;
 }
 
@@ -504,14 +501,15 @@ ImageBufAlgo::make_kernel(string_view name, float width, float height,
             p[0] = (*filter)((float)p.x(), (float)p.y());
     } else if (name == "binomial") {
         // Binomial filter
-        float* wfilter = ALLOCA(float, width);
+        float* wfilter = OIIO_ALLOCA(float, width);
         for (int i = 0; i < width; ++i)
             wfilter[i] = binomial(width - 1, i);
-        float* hfilter = (height == width) ? wfilter : ALLOCA(float, height);
+        float* hfilter = (height == width) ? wfilter
+                                           : OIIO_ALLOCA(float, height);
         if (height != width)
             for (int i = 0; i < height; ++i)
                 hfilter[i] = binomial(height - 1, i);
-        float* dfilter = ALLOCA(float, depth);
+        float* dfilter = OIIO_ALLOCA(float, depth);
         if (depth == 1)
             dfilter[0] = 1;
         else
@@ -531,7 +529,7 @@ ImageBufAlgo::make_kernel(string_view name, float width, float height,
         float val = normalize ? 1.0f / ((w * h * d)) : 1.0f;
         for (ImageBuf::Iterator<float> p(dst); !p.done(); ++p)
             p[0] = val;
-        dst.error("Unknown kernel \"%s\" %gx%g", name, width, height);
+        dst.errorf("Unknown kernel \"%s\" %gx%g", name, width, height);
         return dst;
     }
     if (normalize) {
@@ -584,11 +582,11 @@ ImageBufAlgo::unsharp_mask(ImageBuf& dst, const ImageBuf& src,
     } else {
         ImageBuf K;
         if (!make_kernel(K, kernel, width, width)) {
-            dst.error("%s", K.geterror());
+            dst.errorf("%s", K.geterror());
             return false;
         }
         if (!convolve(Blurry, src, K, true, roi, nthreads)) {
-            dst.error("%s", Blurry.geterror());
+            dst.errorf("%s", Blurry.geterror());
             return false;
         }
     }
@@ -606,7 +604,7 @@ ImageBufAlgo::unsharp_mask(ImageBuf& dst, const ImageBuf& src,
     if (ok)
         ok = mul(Diff, Diff, contrast, roi, nthreads);
     if (!ok) {
-        dst.error("%s", Diff.geterror());
+        dst.errorf("%s", Diff.geterror());
         return false;
     }
 
@@ -627,7 +625,7 @@ ImageBufAlgo::unsharp_mask(const ImageBuf& src, string_view kernel, float width,
     bool ok = unsharp_mask(result, src, kernel, width, contrast, threshold, roi,
                            nthreads);
     if (!ok && !result.has_error())
-        result.error("ImageBufAlgo::unsharp_mask() error");
+        result.errorf("ImageBufAlgo::unsharp_mask() error");
     return result;
 }
 
@@ -644,7 +642,7 @@ ImageBufAlgo::laplacian(ImageBuf& dst, const ImageBuf& src, ROI roi,
 
     ImageBuf K;
     if (!make_kernel(K, "laplacian", 3, 3)) {
-        dst.error("%s", K.geterror());
+        dst.errorf("%s", K.geterror());
         return false;
     }
     // K.write ("K.exr");
@@ -660,7 +658,7 @@ ImageBufAlgo::laplacian(const ImageBuf& src, ROI roi, int nthreads)
     ImageBuf result;
     bool ok = laplacian(result, src, roi, nthreads);
     if (!ok && !result.has_error())
-        result.error("ImageBufAlgo::laplacian() error");
+        result.errorf("ImageBufAlgo::laplacian() error");
     return result;
 }
 
@@ -739,7 +737,7 @@ ImageBufAlgo::median_filter(const ImageBuf& src, int width, int height, ROI roi,
     ImageBuf result;
     bool ok = median_filter(result, src, width, height, roi, nthreads);
     if (!ok && !result.has_error())
-        result.error("ImageBufAlgo::median_filter() error");
+        result.errorf("ImageBufAlgo::median_filter() error");
     return result;
 }
 
@@ -821,7 +819,7 @@ ImageBufAlgo::dilate(const ImageBuf& src, int width, int height, ROI roi,
     ImageBuf result;
     bool ok = dilate(result, src, width, height, roi, nthreads);
     if (!ok && !result.has_error())
-        result.error("ImageBufAlgo::dilate() error");
+        result.errorf("ImageBufAlgo::dilate() error");
     return result;
 }
 
@@ -852,7 +850,7 @@ ImageBufAlgo::erode(const ImageBuf& src, int width, int height, ROI roi,
     ImageBuf result;
     bool ok = erode(result, src, width, height, roi, nthreads);
     if (!ok && !result.has_error())
-        result.error("ImageBufAlgo::erode() error");
+        result.errorf("ImageBufAlgo::erode() error");
     return result;
 }
 
@@ -898,7 +896,7 @@ ImageBufAlgo::fft(ImageBuf& dst, const ImageBuf& src, ROI roi, int nthreads)
 {
     pvt::LoggedTimer logtime("IBA::fft");
     if (src.spec().depth > 1) {
-        dst.error("ImageBufAlgo::fft does not support volume images");
+        dst.errorf("ImageBufAlgo::fft does not support volume images");
         return false;
     }
     if (!roi.defined())
@@ -938,7 +936,7 @@ ImageBufAlgo::fft(ImageBuf& dst, const ImageBuf& src, ROI roi, int nthreads)
         zero(A, r);
     }
     if (!ImageBufAlgo::paste(A, 0, 0, 0, 0, src, roi, nthreads)) {
-        dst.error("%s", A.geterror());
+        dst.errorf("%s", A.geterror());
         return false;
     }
 
@@ -969,11 +967,11 @@ ImageBufAlgo::ifft(ImageBuf& dst, const ImageBuf& src, ROI roi, int nthreads)
 {
     pvt::LoggedTimer logtime("IBA::ifft");
     if (src.nchannels() != 2 || src.spec().format != TypeDesc::FLOAT) {
-        dst.error("ifft can only be done on 2-channel float images");
+        dst.errorf("ifft can only be done on 2-channel float images");
         return false;
     }
     if (src.spec().depth > 1) {
-        dst.error("ImageBufAlgo::ifft does not support volume images");
+        dst.errorf("ImageBufAlgo::ifft does not support volume images");
         return false;
     }
 
@@ -1031,7 +1029,7 @@ ImageBufAlgo::fft(const ImageBuf& src, ROI roi, int nthreads)
     ImageBuf result;
     bool ok = fft(result, src, roi, nthreads);
     if (!ok && !result.has_error())
-        result.error("ImageBufAlgo::fft() error");
+        result.errorf("ImageBufAlgo::fft() error");
     return result;
 }
 
@@ -1042,7 +1040,7 @@ ImageBufAlgo::ifft(const ImageBuf& src, ROI roi, int nthreads)
     ImageBuf result;
     bool ok = ifft(result, src, roi, nthreads);
     if (!ok && !result.has_error())
-        result.error("ImageBufAlgo::ifft() error");
+        result.errorf("ImageBufAlgo::ifft() error");
     return result;
 }
 
@@ -1079,7 +1077,7 @@ complex_to_polar_impl(ImageBuf& R, const ImageBuf& A, ROI roi, int nthreads)
             float imag  = a[1];
             float phase = std::atan2(imag, real);
             if (phase < 0.0f)
-                phase += float(M_TWO_PI);
+                phase += float(2.0 * M_PI);
             r[0] = hypotf(real, imag);
             r[1] = phase;
         }
@@ -1095,14 +1093,14 @@ ImageBufAlgo::polar_to_complex(ImageBuf& dst, const ImageBuf& src, ROI roi,
 {
     pvt::LoggedTimer logtime("IBA::polar_to_complex");
     if (src.nchannels() != 2) {
-        dst.error("polar_to_complex can only be done on 2-channel");
+        dst.errorf("polar_to_complex can only be done on 2-channel");
         return false;
     }
 
     if (!IBAprep(roi, &dst, &src))
         return false;
     if (dst.nchannels() != 2) {
-        dst.error("polar_to_complex can only be done on 2-channel");
+        dst.errorf("polar_to_complex can only be done on 2-channel");
         return false;
     }
     bool ok;
@@ -1120,7 +1118,7 @@ ImageBufAlgo::polar_to_complex(const ImageBuf& src, ROI roi, int nthreads)
     ImageBuf result;
     bool ok = polar_to_complex(result, src, roi, nthreads);
     if (!ok && !result.has_error())
-        result.error("ImageBufAlgo::polar_to_complex() error");
+        result.errorf("ImageBufAlgo::polar_to_complex() error");
     return result;
 }
 
@@ -1132,14 +1130,14 @@ ImageBufAlgo::complex_to_polar(ImageBuf& dst, const ImageBuf& src, ROI roi,
 {
     pvt::LoggedTimer logtime("IBA::complex_to_polar");
     if (src.nchannels() != 2) {
-        dst.error("complex_to_polar can only be done on 2-channel");
+        dst.errorf("complex_to_polar can only be done on 2-channel");
         return false;
     }
 
     if (!IBAprep(roi, &dst, &src))
         return false;
     if (dst.nchannels() != 2) {
-        dst.error("complex_to_polar can only be done on 2-channel");
+        dst.errorf("complex_to_polar can only be done on 2-channel");
         return false;
     }
     bool ok;
@@ -1157,7 +1155,7 @@ ImageBufAlgo::complex_to_polar(const ImageBuf& src, ROI roi, int nthreads)
     ImageBuf result;
     bool ok = complex_to_polar(result, src, roi, nthreads);
     if (!ok && !result.has_error())
-        result.error("ImageBufAlgo::complex_to_polar() error");
+        result.errorf("ImageBufAlgo::complex_to_polar() error");
     return result;
 }
 
@@ -1220,7 +1218,7 @@ ImageBufAlgo::fillholes_pushpull(ImageBuf& dst, const ImageBuf& src, ROI roi,
         ImageBufAlgo::resize(*small, *pyramid.back(), "triangle");
         divide_by_alpha(*small, get_roi(smallspec), nthreads);
         pyramid.emplace_back(small);
-        //debug small->save();
+        // small->write(Strutil::sprintf("push%04d.exr", small->spec().width));
     }
 
     // Now pull back up the pyramid by doing an alpha composite of level
@@ -1233,7 +1231,7 @@ ImageBufAlgo::fillholes_pushpull(ImageBuf& dst, const ImageBuf& src, ROI roi,
         ImageBuf blowup(big.spec());
         ImageBufAlgo::resize(blowup, small, "triangle");
         ImageBufAlgo::over(big, big, blowup);
-        //debug big.save (Strutil::sprintf("after%d.exr", i));
+        // big.write(Strutil::sprintf("pull%04d.exr", big.spec().width));
     }
 
     // Now copy the completed base layer of the pyramid back to the
@@ -1251,7 +1249,7 @@ ImageBufAlgo::fillholes_pushpull(const ImageBuf& src, ROI roi, int nthreads)
     ImageBuf result;
     bool ok = fillholes_pushpull(result, src, roi, nthreads);
     if (!ok && !result.has_error())
-        result.error("ImageBufAlgo::fillholes_pushpull() error");
+        result.errorf("ImageBufAlgo::fillholes_pushpull() error");
     return result;
 }
 
