@@ -1,32 +1,6 @@
-/*
-  Copyright 2008 Larry Gritz and the other authors and contributors.
-  All Rights Reserved.
-
-  Redistribution and use in source and binary forms, with or without
-  modification, are permitted provided that the following conditions are
-  met:
-  * Redistributions of source code must retain the above copyright
-    notice, this list of conditions and the following disclaimer.
-  * Redistributions in binary form must reproduce the above copyright
-    notice, this list of conditions and the following disclaimer in the
-    documentation and/or other materials provided with the distribution.
-  * Neither the name of the software's owners nor the names of its
-    contributors may be used to endorse or promote products derived from
-    this software without specific prior written permission.
-  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-
-  (This is the Modified BSD License)
-*/
+// Copyright 2008-present Contributors to the OpenImageIO project.
+// SPDX-License-Identifier: BSD-3-Clause
+// https://github.com/OpenImageIO/oiio/blob/master/LICENSE.md
 
 #include <cstdio>
 #include <cstdlib>
@@ -40,6 +14,7 @@
 #include <OpenImageIO/fmath.h>
 #include <OpenImageIO/hash.h>
 #include <OpenImageIO/imageio.h>
+#include <OpenImageIO/optparser.h>
 #include <OpenImageIO/parallel.h>
 #include <OpenImageIO/strutil.h>
 #include <OpenImageIO/sysutil.h>
@@ -76,12 +51,13 @@ std::string extension_list;      // list of all extensions for all formats
 std::string library_list;        // list of all libraries for all formats
 static const char* oiio_debug_env = getenv("OPENIMAGEIO_DEBUG");
 #ifdef NDEBUG
-int oiio_print_debug(oiio_debug_env ? atoi(oiio_debug_env) : 0);
+int oiio_print_debug(oiio_debug_env ? Strutil::stoi(oiio_debug_env) : 0);
 #else
-int oiio_print_debug(oiio_debug_env ? atoi(oiio_debug_env) : 1);
+int oiio_print_debug(oiio_debug_env ? Strutil::stoi(oiio_debug_env) : 1);
 #endif
 int oiio_log_times = Strutil::from_string<int>(
     Sysutil::getenv("OPENIMAGEIO_LOG_TIMES"));
+std::vector<float> oiio_missingcolor;
 }  // namespace pvt
 
 using namespace pvt;
@@ -98,7 +74,7 @@ public:
     spin_mutex mutex;
     std::map<std::string, std::pair<double, size_t>> timing_map;
 
-    TimingLog() {}
+    TimingLog() noexcept {}
 
     // Destructor prints the timing report if oiio_log_times >= 2
     ~TimingLog()
@@ -143,6 +119,27 @@ public:
 };
 static TimingLog timing_log;
 
+
+
+// Pipe-fitting class to set global options, for the sake of optparser.
+struct GlobalOptSetter {
+public:
+    template<typename T> bool attribute(string_view name, T val) const
+    {
+        return OIIO::attribute(name, val);
+    }
+};
+
+
+// Trick to force get the "OPENIMAGEIO_OPTIONS" env var on startup.
+bool force_global_opts = []() {
+    auto options = Sysutil::getenv("OPENIMAGEIO_OPTIONS");
+    // std::cout << "OPTIONS: '" << options << "'\n";
+    if (!options.empty())
+        attribute("options", options);
+    return true;
+}();
+
 }  // namespace
 
 
@@ -283,6 +280,10 @@ pvt::log_time(string_view key, const Timer& timer)
 bool
 attribute(string_view name, TypeDesc type, const void* val)
 {
+    if (name == "options" && type == TypeDesc::STRING) {
+        GlobalOptSetter gos;
+        return optparser(gos, *(const char**)val);
+    }
     if (name == "threads" && type == TypeInt) {
         int ot = Imath::clamp(*(const int*)val, 0, maxthreads);
         if (ot == 0)
@@ -320,6 +321,22 @@ attribute(string_view name, TypeDesc type, const void* val)
         oiio_log_times = *(const int*)val;
         return true;
     }
+    if (name == "missingcolor" && type.basetype == TypeDesc::FLOAT) {
+        // missingcolor as float array
+        oiio_missingcolor.clear();
+        oiio_missingcolor.reserve(type.basevalues());
+        int n = type.basevalues();
+        for (int i = 0; i < n; ++i)
+            oiio_missingcolor[i] = ((const float*)val)[i];
+        return true;
+    }
+    if (name == "missingcolor" && type == TypeString) {
+        // missingcolor as string
+        oiio_missingcolor = Strutil::extract_from_list_string<float>(
+            *(const char**)val);
+        return true;
+    }
+
     return false;
 }
 
@@ -407,6 +424,22 @@ getattribute(string_view name, TypeDesc type, void* val)
         *(int*)val = int(Sysutil::memory_used(true) >> 20);
         return true;
     }
+    if (name == "missingcolor" && type.basetype == TypeDesc::FLOAT
+        && oiio_missingcolor.size()) {
+        // missingcolor as float array
+        int n  = type.basevalues();
+        int nm = std::min(int(oiio_missingcolor.size()), n);
+        for (int i = 0; i < nm; ++i)
+            ((float*)val)[i] = oiio_missingcolor[i];
+        for (int i = nm; i < n; ++i)
+            ((float*)val)[i] = 0.0f;
+        return true;
+    }
+    if (name == "missingcolor" && type == TypeString) {
+        // missingcolor as string
+        *(ustring*)val = ustring(Strutil::join(oiio_missingcolor, ","));
+        return true;
+    }
     return false;
 }
 
@@ -606,8 +639,8 @@ pvt::parallel_convert_from_float(const float* src, void* dst, size_t nvals,
 
 
 bool
-convert_types(TypeDesc src_type, const void* src, TypeDesc dst_type, void* dst,
-              int n)
+convert_pixel_values(TypeDesc src_type, const void* src, TypeDesc dst_type,
+                     void* dst, int n)
 {
     // If no conversion is necessary, just memcpy
     if ((src_type == dst_type || dst_type.basetype == TypeDesc::UNKNOWN)) {
@@ -628,7 +661,7 @@ convert_types(TypeDesc src_type, const void* src, TypeDesc dst_type, void* dst,
     if (src_type != TypeFloat) {
         // If src is also not float, convert through an intermediate buffer
         if (n <= 4096)  // If < 16k, use the stack
-            buf = ALLOCA(float, n);
+            buf = OIIO_ALLOCA(float, n);
         else {
             tmp.reset(new float[n]);  // Freed when tmp exists its scope
             buf = tmp.get();
@@ -688,15 +721,15 @@ convert_image(int nchannels, int width, int height, int depth, const void* src,
                 // Special case: pixels within each row are contiguous
                 // in both src and dst and we're copying all channels.
                 // Be efficient by converting each scanline as a single
-                // unit.  (Note that within convert_types, a memcpy will
-                // be used if the formats are identical.)
-                result &= convert_types(src_type, f, dst_type, t,
-                                        nchannels * width);
+                // unit.  (Note that within convert_pixel_values, a memcpy
+                // will be used if the formats are identical.)
+                result &= convert_pixel_values(src_type, f, dst_type, t,
+                                               nchannels * width);
             } else {
                 // General case -- anything goes with strides.
                 for (int x = 0; x < width; ++x) {
-                    result &= convert_types(src_type, f, dst_type, t,
-                                            nchannels);
+                    result &= convert_pixel_values(src_type, f, dst_type, t,
+                                                   nchannels);
                     f += src_xstride;
                     t += dst_xstride;
                 }
