@@ -1,32 +1,6 @@
-/*
-  Copyright 2013 Larry Gritz and the other authors and contributors.
-  All Rights Reserved.
-
-  Redistribution and use in source and binary forms, with or without
-  modification, are permitted provided that the following conditions are
-  met:
-  * Redistributions of source code must retain the above copyright
-    notice, this list of conditions and the following disclaimer.
-  * Redistributions in binary form must reproduce the above copyright
-    notice, this list of conditions and the following disclaimer in the
-    documentation and/or other materials provided with the distribution.
-  * Neither the name of the software's owners nor the names of its
-    contributors may be used to endorse or promote products derived from
-    this software without specific prior written permission.
-  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-
-  (This is the Modified BSD License)
-*/
+// Copyright 2008-present Contributors to the OpenImageIO project.
+// SPDX-License-Identifier: BSD-3-Clause
+// https://github.com/OpenImageIO/oiio/blob/master/LICENSE.md
 
 #include <cmath>
 #include <cstdio>
@@ -37,8 +11,6 @@
 #include <memory>
 #include <sstream>
 
-#include <boost/version.hpp>
-
 #include <OpenEXR/ImathMatrix.h>
 #include <OpenEXR/half.h>
 
@@ -177,7 +149,8 @@ interppixel_NDC_clamped(const ImageBuf& buf, float x, float y, float* pixel,
     y      = static_cast<float>(fy) + y * static_cast<float>(fh);
 
     int n     = buf.spec().nchannels;
-    float *p0 = ALLOCA(float, 4 * n), *p1 = p0 + n, *p2 = p1 + n, *p3 = p2 + n;
+    float *p0 = OIIO_ALLOCA(float, 4 * n), *p1 = p0 + n, *p2 = p1 + n,
+          *p3 = p2 + n;
 
     x -= 0.5f;
     y -= 0.5f;
@@ -238,7 +211,7 @@ resize_block_(ImageBuf& dst, const ImageBuf& src, ROI roi, bool envlatlmode)
            || srcspec.z + srcspec.depth < srcspec.full_z + srcspec.full_depth);
 
     const ImageSpec& dstspec(dst.spec());
-    float* pel    = ALLOCA(float, dstspec.nchannels);
+    float* pel    = OIIO_ALLOCA(float, dstspec.nchannels);
     float xoffset = (float)dstspec.full_x;
     float yoffset = (float)dstspec.full_y;
     float xscale  = 1.0f / (float)dstspec.full_width;
@@ -362,7 +335,7 @@ check_nan_block(const ImageBuf& src, ROI roi, int& found_nonfinite)
 {
     int x0 = roi.xbegin, x1 = roi.xend, y0 = roi.ybegin, y1 = roi.yend;
     const ImageSpec& spec(src.spec());
-    float* pel = ALLOCA(float, spec.nchannels);
+    float* pel = OIIO_ALLOCA(float, spec.nchannels);
     for (int y = y0; y < y1; ++y) {
         for (int x = x0; x < x1; ++x) {
             src.getpixel(x, y, pel);
@@ -411,7 +384,7 @@ lightprobe_to_envlatl(ImageBuf& dst, const ImageBuf& src, bool y_is_up,
         int nchannels = dstspec.nchannels;
         ASSERT(dstspec.format == TypeDesc::FLOAT);
 
-        float* pixel = ALLOCA(float, nchannels);
+        float* pixel = OIIO_ALLOCA(float, nchannels);
         float dw = dstspec.width, dh = dstspec.height;
         for (ImageBuf::Iterator<float> d(dst, roi); !d.done(); ++d) {
             Imath::V3f V = latlong_to_dir((d.x() + 0.5f) / dw,
@@ -543,8 +516,8 @@ static void
 fix_latl_edges(ImageBuf& buf)
 {
     int n        = buf.nchannels();
-    float* left  = ALLOCA(float, n);
-    float* right = ALLOCA(float, n);
+    float* left  = OIIO_ALLOCA(float, n);
+    float* right = OIIO_ALLOCA(float, n);
 
     // Make the whole first and last row be solid, since they are exactly
     // on the pole
@@ -639,22 +612,35 @@ write_mipmap(ImageBufAlgo::MakeTextureMode mode, std::shared_ptr<ImageBuf>& img,
         return false;
     }
 
-    if (!mipmap && !strcmp(out->format_name(), "openexr")) {
-        // Send hint to OpenEXR driver that we won't specify a MIPmap
-        outspec.attribute("openexr:levelmode", 0 /* ONE_LEVEL */);
-    }
-
-    if (mipmap && !strcmp(out->format_name(), "openexr")) {
-        outspec.attribute("openexr:roundingmode", 0 /* ROUND_DOWN */);
-    }
-
-    // OpenEXR always uses border sampling for environment maps
+    bool verbose = configspec.get_int_attribute("maketx:verbose") != 0;
     bool src_samples_border = false;
-    if (envlatlmode && !strcmp(out->format_name(), "openexr")) {
-        src_samples_border = true;
-        outspec.attribute("oiio:updirection", "y");
-        outspec.attribute("oiio:sampleborder", 1);
+
+    // Some special constraints for OpenEXR
+    if (!strcmp(out->format_name(), "openexr")) {
+        // Always use "round down" mode
+        outspec.attribute("openexr:roundingmode", 0 /* ROUND_DOWN */);
+        if (!mipmap) {
+            // Send hint to OpenEXR driver that we won't specify a MIPmap
+            outspec.attribute("openexr:levelmode", 0 /* ONE_LEVEL */);
+        }
+        // OpenEXR always uses border sampling for environment maps
+        if (envlatlmode) {
+            src_samples_border = true;
+            outspec.attribute("oiio:updirection", "y");
+            outspec.attribute("oiio:sampleborder", 1);
+        }
+        // For single channel images, dwaa/b compression only seems to work
+        // reliably when size > 16 and size is a power of two. Bug?
+        // FIXME: watch future OpenEXR releases to see if this gets fixed.
+        if (outspec.nchannels == 1
+            && Strutil::istarts_with(outspec["compression"].get(), "dwa")) {
+            outspec.attribute("compression", "zip");
+            if (verbose)
+                outstream
+                    << "WARNING: Changing unsupported DWA compression for this case to zip.\n";
+        }
     }
+
     if (envlatlmode && src_samples_border)
         fix_latl_edges(*img);
 
@@ -681,7 +667,6 @@ write_mipmap(ImageBufAlgo::MakeTextureMode mode, std::shared_ptr<ImageBuf>& img,
     }
 
     // Write out the image
-    bool verbose = configspec.get_int_attribute("maketx:verbose") != 0;
     if (verbose) {
         outstream << "  Writing file: " << outputfilename << std::endl;
         outstream << "  Filter \"" << filtername << "\"\n";
@@ -1507,6 +1492,8 @@ make_texture_impl(ImageBufAlgo::MakeTextureMode mode, const ImageBuf* input,
         }
 
         if (isConstantColor) {
+            if (constantColor.size() < 3)
+                constantColor.resize(3, constantColor[0]);
             if (!ImageBufAlgo::colorconvert(
                     &constantColor[0], static_cast<int>(constantColor.size()),
                     processor.get(), unpremult)) {
@@ -1517,6 +1504,8 @@ make_texture_impl(ImageBufAlgo::MakeTextureMode mode, const ImageBuf* input,
         }
 
         if (compute_average_color) {
+            if (pixel_stats.avg.size() < 3)
+                pixel_stats.avg.resize(3, pixel_stats.avg[0]);
             if (!ImageBufAlgo::colorconvert(&pixel_stats.avg[0],
                                             static_cast<int>(
                                                 pixel_stats.avg.size()),
@@ -1546,8 +1535,8 @@ make_texture_impl(ImageBufAlgo::MakeTextureMode mode, const ImageBuf* input,
 
     // Handle resize to power of two, if called for
     if (configspec.get_int_attribute("maketx:resize") && !shadowmode) {
-        dstspec.width       = pow2roundup(dstspec.width);
-        dstspec.height      = pow2roundup(dstspec.height);
+        dstspec.width       = ceil2(dstspec.width);
+        dstspec.height      = ceil2(dstspec.height);
         dstspec.full_width  = dstspec.width;
         dstspec.full_height = dstspec.height;
     }
@@ -1679,47 +1668,33 @@ make_texture_impl(ImageBufAlgo::MakeTextureMode mode, const ImageBuf* input,
     STATUS("SHA-1 hash", stat_hashtime);
 
     if (isConstantColor) {
-        std::ostringstream os;             // Emulate a JSON array
-        os.imbue(std::locale::classic());  // Force "C" locale with '.' decimal
-        for (int i = 0; i < dstspec.nchannels; ++i) {
-            if (i != 0)
-                os << ",";
-            os << (i < (int)constantColor.size() ? constantColor[i] : 0.0f);
-        }
+        std::string colstr = Strutil::join(constantColor, ",",
+                                           dstspec.nchannels);
         if (out->supports("arbitrary_metadata")) {
-            dstspec.attribute("oiio:ConstantColor", os.str());
+            dstspec.attribute("oiio:ConstantColor", colstr);
         } else {
-            if (desc.length())
-                desc += " ";
-            desc += "oiio:ConstantColor=";
-            desc += os.str();
+            desc += Strutil::sprintf("%soiio:ConstantColor=%s",
+                                     desc.length() ? " " : "", colstr);
             updatedDesc = true;
         }
         if (verbose)
-            outstream << "  ConstantColor: " << os.str() << std::endl;
+            outstream << "  ConstantColor: " << colstr << std::endl;
     }
 
     if (compute_average_color) {
-        std::ostringstream os;             // Emulate a JSON array
-        os.imbue(std::locale::classic());  // Force "C" locale with '.' decimal
-        for (int i = 0; i < dstspec.nchannels; ++i) {
-            if (i != 0)
-                os << ",";
-            os << (i < (int)pixel_stats.avg.size() ? pixel_stats.avg[i] : 0.0f);
-        }
+        std::string avgstr = Strutil::join(pixel_stats.avg, ",",
+                                           dstspec.nchannels);
         if (out->supports("arbitrary_metadata")) {
-            dstspec.attribute("oiio:AverageColor", os.str());
+            dstspec.attribute("oiio:AverageColor", avgstr);
         } else {
             // if arbitrary metadata is not supported, cram it into the
             // ImageDescription.
-            if (desc.length())
-                desc += " ";
-            desc += "oiio:AverageColor=";
-            desc += os.str();
+            desc += Strutil::sprintf("%soiio:AverageColor=%s",
+                                     desc.length() ? " " : "", avgstr);
             updatedDesc = true;
         }
         if (verbose)
-            outstream << "  AverageColor: " << os.str() << std::endl;
+            outstream << "  AverageColor: " << avgstr << std::endl;
     }
 
     if (updatedDesc) {
@@ -1736,7 +1711,7 @@ make_texture_impl(ImageBufAlgo::MakeTextureMode mode, const ImageBuf* input,
     double misc_time_5 = alltime.lap();
     STATUS("misc4", misc_time_5);
 
-    // Write out, and compute, the mipmap levels for the speicifed image
+    // Write out, and compute, the mipmap levels for the specified image
     bool nomipmap = configspec.get_int_attribute("maketx:nomipmap") != 0;
     bool ok = write_mipmap(mode, toplevel, dstspec, tmpfilename, out.get(),
                            out_dataformat, !shadowmode && !nomipmap, filtername,
