@@ -1,32 +1,6 @@
-/*
-  Copyright 2011 Larry Gritz and the other authors and contributors.
-  All Rights Reserved.
-
-  Redistribution and use in source and binary forms, with or without
-  modification, are permitted provided that the following conditions are
-  met:
-  * Redistributions of source code must retain the above copyright
-    notice, this list of conditions and the following disclaimer.
-  * Redistributions in binary form must reproduce the above copyright
-    notice, this list of conditions and the following disclaimer in the
-    documentation and/or other materials provided with the distribution.
-  * Neither the name of the software's owners nor the names of its
-    contributors may be used to endorse or promote products derived from
-    this software without specific prior written permission.
-  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-
-  (This is the Modified BSD License)
-*/
+// Copyright 2008-present Contributors to the OpenImageIO project.
+// SPDX-License-Identifier: BSD-3-Clause
+// https://github.com/OpenImageIO/oiio/blob/master/LICENSE.md
 
 #include <iostream>
 #include <vector>
@@ -106,19 +80,19 @@ test_int_helpers()
     }
     OIIO_CHECK_ASSERT(ispow2((unsigned int)0));
 
-    // pow2roundup
-    OIIO_CHECK_EQUAL(pow2roundup(4), 4);
-    OIIO_CHECK_EQUAL(pow2roundup(5), 8);
-    OIIO_CHECK_EQUAL(pow2roundup(6), 8);
-    OIIO_CHECK_EQUAL(pow2roundup(7), 8);
-    OIIO_CHECK_EQUAL(pow2roundup(8), 8);
+    // ceil2
+    OIIO_CHECK_EQUAL(ceil2(4), 4);
+    OIIO_CHECK_EQUAL(ceil2(5), 8);
+    OIIO_CHECK_EQUAL(ceil2(6), 8);
+    OIIO_CHECK_EQUAL(ceil2(7), 8);
+    OIIO_CHECK_EQUAL(ceil2(8), 8);
 
-    // pow2rounddown
-    OIIO_CHECK_EQUAL(pow2rounddown(4), 4);
-    OIIO_CHECK_EQUAL(pow2rounddown(5), 4);
-    OIIO_CHECK_EQUAL(pow2rounddown(6), 4);
-    OIIO_CHECK_EQUAL(pow2rounddown(7), 4);
-    OIIO_CHECK_EQUAL(pow2rounddown(8), 8);
+    // floor2
+    OIIO_CHECK_EQUAL(floor2(4), 4);
+    OIIO_CHECK_EQUAL(floor2(5), 4);
+    OIIO_CHECK_EQUAL(floor2(6), 4);
+    OIIO_CHECK_EQUAL(floor2(7), 4);
+    OIIO_CHECK_EQUAL(floor2(8), 8);
 
     // round_to_multiple
     OIIO_CHECK_EQUAL(round_to_multiple(0, 5), 0);
@@ -144,6 +118,10 @@ test_int_helpers()
     OIIO_CHECK_EQUAL(round_to_multiple_of_pow2(size_t(3), size_t(4)), 4);
     OIIO_CHECK_EQUAL(round_to_multiple_of_pow2(size_t(4), size_t(4)), 4);
     OIIO_CHECK_EQUAL(round_to_multiple_of_pow2(size_t(5), size_t(4)), 8);
+
+    OIIO_CHECK_EQUAL(rotl(uint32_t(0x12345678), 4), uint32_t(0x23456781));
+    OIIO_CHECK_EQUAL(rotl(uint64_t(0x123456789abcdef0ULL), 4),
+                     uint64_t(0x23456789abcdef01ULL));
 }
 
 
@@ -278,6 +256,67 @@ test_bit_range_convert()
 
 
 
+void
+test_packbits()
+{
+    std::cout << "test_convert_pack_bits\n";
+
+    {
+        unsigned char foo[3] = { 0, 0, 0 };
+        unsigned char* fp    = foo;
+        int fpf              = 0;
+        bitstring_add_n_bits(fp, fpf, 1, 4);
+        bitstring_add_n_bits(fp, fpf, 2, 8);
+        bitstring_add_n_bits(fp, fpf, 0xffff, 10);
+        // result should be 0x10 0x2f 0xfc
+        Strutil::printf("  bitstring_add_n_bits results %02x %02x %02x\n",
+                        foo[0], foo[1], foo[2]);
+        OIIO_CHECK_EQUAL(foo[0], 0x10);
+        OIIO_CHECK_EQUAL(foo[1], 0x2f);
+        OIIO_CHECK_EQUAL(foo[2], 0xfc);
+    }
+    {
+        unsigned char foo[4] = { 0, 0, 0, 0 };
+        unsigned char* fp    = foo;
+        int fpf              = 0;
+        bitstring_add_n_bits(fp, fpf, 1023, 10);
+        bitstring_add_n_bits(fp, fpf, 0, 10);
+        bitstring_add_n_bits(fp, fpf, 1023, 10);
+        // result should be 1111111111 0000000000 1111111111 00
+        //                     f   f    c   0   0    f   f    c
+        Strutil::printf("  bitstring_add_n_bits results %02x %02x %02x %02x\n",
+                        foo[0], foo[1], foo[2], foo[3]);
+        OIIO_CHECK_EQUAL(foo[0], 0xff);
+        OIIO_CHECK_EQUAL(foo[1], 0xc0);
+        OIIO_CHECK_EQUAL(foo[2], 0x0f);
+        OIIO_CHECK_EQUAL(foo[3], 0xfc);
+    }
+
+    const uint16_t u16vals[8] = { 1, 1, 1, 1, 1, 1, 1, 1 };
+    uint16_t u10[5]           = { 255, 255, 255, 255, 255 };
+    Strutil::printf(
+        " in 16 bit values: %04x %04x %04x %04x %04x %04x %04x %04x\n",
+        u16vals[0], u16vals[1], u16vals[2], u16vals[3], u16vals[4], u16vals[5],
+        u16vals[6], u16vals[7]);
+    bit_pack(cspan<uint16_t>(u16vals, 8), u10, 10);
+    Strutil::printf(
+        " packed to 10 bits, as 16 bit values: %04x %04x %04x %04x %04x\n",
+        u10[0], u10[1], u10[2], u10[3], u10[4]);
+    uint16_t u16[8];
+    bit_unpack(8, (const unsigned char*)u10, 10, u16);
+    Strutil::printf(
+        " unpacked back to 16 bits: %04x %04x %04x %04x %04x %04x %04x %04x\n",
+        u16[0], u16[1], u16[2], u16[3], u16[4], u16[5], u16[6], u16[7]);
+    // Before: 00000000 00000001  00000000 00000001  00000000 00000001...
+    // After:  00000000 01000000  00010000 00000100  00000001 00000000  01000000 00010000  00000100 00000001
+    //       =  00 40  10 04  01 00  40 10  04 01
+    // as little endian 16 bit:  4000 0410 0001 1040 0104
+    for (size_t i = 0; i < 8; ++i)
+        OIIO_CHECK_EQUAL(u16vals[i], u16[i]);
+}
+
+
+
 static void
 test_interpolate_linear()
 {
@@ -439,6 +478,7 @@ main(int argc, char* argv[])
     //    test_convert_type<float,unsigned short> ();
 
     test_bit_range_convert();
+    test_packbits();
 
     test_interpolate_linear();
 
