@@ -1,32 +1,6 @@
-/*
-  Copyright 2008 Larry Gritz and the other authors and contributors.
-  All Rights Reserved.
-
-  Redistribution and use in source and binary forms, with or without
-  modification, are permitted provided that the following conditions are
-  met:
-  * Redistributions of source code must retain the above copyright
-    notice, this list of conditions and the following disclaimer.
-  * Redistributions in binary form must reproduce the above copyright
-    notice, this list of conditions and the following disclaimer in the
-    documentation and/or other materials provided with the distribution.
-  * Neither the name of the software's owners nor the names of its
-    contributors may be used to endorse or promote products derived from
-    this software without specific prior written permission.
-  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-
-  (This is the Modified BSD License)
-*/
+// Copyright 2008-present Contributors to the OpenImageIO project.
+// SPDX-License-Identifier: BSD-3-Clause
+// https://github.com/OpenImageIO/oiio/blob/master/LICENSE.md
 
 
 #include <cmath>
@@ -56,13 +30,33 @@
 #endif
 
 
+// We use the public domain stb implementation of vsnprintf because
+// not all platforms support a locale-independent version of vsnprintf.
+// See: https://github.com/nothings/stb/blob/master/stb_sprintf.h
+#define STB_SPRINTF_DECORATE(name) oiio_stbsp_##name
+#define STB_SPRINTF_IMPLEMENTATION 1
+#if defined(__GNUG__) && !defined(__clang__)
+#    pragma GCC diagnostic ignored "-Wstrict-aliasing"
+#    define STBI__ASAN OIIO_NO_SANITIZE_ADDRESS
+#endif
+#include "stb_sprintf.h"
+
 
 OIIO_NAMESPACE_BEGIN
 
 
 namespace {
 static std::mutex output_mutex;
-};
+
+// On systems that support it, get a location independent locale.
+#if defined(__linux__) || defined(__APPLE__) || defined(__FreeBSD__)           \
+    || defined(__FreeBSD_kernel__) || defined(__GLIBC__)
+static locale_t c_loc = newlocale(LC_ALL_MASK, "C", nullptr);
+#elif defined(_WIN32)
+static _locale_t c_loc = _create_locale(LC_ALL, "C");
+#endif
+
+};  // namespace
 
 
 
@@ -172,7 +166,8 @@ Strutil::vsprintf(const char* fmt, va_list ap)
 #else
         apsave = ap;
 #endif
-        int needed = vsnprintf(buf, size, fmt, ap);
+
+        int needed = oiio_stbsp_vsnprintf(buf, size, fmt, ap);
         va_end(ap);
 
         // NB. C99 says vsnprintf returns -1 on an encoding error. Otherwise
@@ -358,20 +353,34 @@ Strutil::unescape_chars(string_view escaped)
 
 
 std::string
-Strutil::wordwrap(string_view src, int columns, int prefix)
+Strutil::wordwrap(string_view src, int columns, int prefix, string_view sep,
+                  string_view presep)
 {
     if (columns < prefix + 20)
         return src;  // give up, no way to make it wrap
     std::ostringstream out;
     columns -= prefix;  // now columns is the real width we have to work with
+    std::string allsep = Strutil::sprintf("%s%s", sep, presep);
     while ((int)src.length() > columns) {
-        // break the string in two
-        size_t breakpoint = src.find_last_of(' ', columns);
+        // Find the last `sep` break before the column limit.
+        size_t breakpoint = src.find_last_of(allsep, columns);
+        // If presep is not empty, find the last presep break before the
+        // column limit and break one character AFTER that, if it's a better
+        // breakpost than the sep break.
+        if (presep.size()) {
+            size_t presep_break = src.find_last_of(presep, columns);
+            if (presep_break >= breakpoint && presep_break < src.size())
+                breakpoint = presep_break + 1;
+        }
+        // If no break was found, punt and hard break at the column limit.
         if (breakpoint == string_view::npos)
             breakpoint = columns;
+        // Copy up to the breakpoint, then newline and prefex blanks.
         out << src.substr(0, breakpoint) << "\n" << std::string(prefix, ' ');
+        // Pick up where we left off
         src = src.substr(breakpoint);
-        while (src[0] == ' ')
+        // Skip any separator characters at the start of the string
+        while (sep.find(src[0]) != string_view::npos)
             src.remove_prefix(1);
     }
     out << src;
@@ -380,6 +389,17 @@ Strutil::wordwrap(string_view src, int columns, int prefix)
 
 
 
+// DEPRECATED(2.0) -- for link compatibility
+namespace Strutil {
+std::string
+wordwrap(string_view src, int columns, int prefix)
+{
+    return wordwrap(src, columns, prefix, " ", "");
+}
+}  // namespace Strutil
+
+
+
 bool
 Strutil::iequals(string_view a, string_view b)
 {
@@ -453,14 +473,14 @@ Strutil::to_upper(std::string& a)
 
 
 bool
-Strutil::StringIEqual::operator()(const char* a, const char* b) const
+Strutil::StringIEqual::operator()(const char* a, const char* b) const noexcept
 {
     return boost::algorithm::iequals(a, b, std::locale::classic());
 }
 
 
 bool
-Strutil::StringILess::operator()(const char* a, const char* b) const
+Strutil::StringILess::operator()(const char* a, const char* b) const noexcept
 {
     return boost::algorithm::ilexicographical_compare(a, b,
                                                       std::locale::classic());
@@ -468,6 +488,28 @@ Strutil::StringILess::operator()(const char* a, const char* b) const
 
 
 
+string_view
+Strutil::lstrip(string_view str, string_view chars)
+{
+    if (chars.empty())
+        chars = string_view(" \t\n\r\f\v", 6);
+    size_t b = str.find_first_not_of(chars);
+    return str.substr(b, string_view::npos);
+}
+
+
+
+string_view
+Strutil::rstrip(string_view str, string_view chars)
+{
+    if (chars.empty())
+        chars = string_view(" \t\n\r\f\v", 6);
+    size_t e = str.find_last_not_of(chars);
+    return e != string_view::npos ? str.substr(0, e + 1) : string_view();
+}
+
+
+
 string_view
 Strutil::strip(string_view str, string_view chars)
 {
@@ -609,7 +651,7 @@ Strutil::replace(string_view str, string_view pattern, string_view replacement,
 
 #ifdef _WIN32
 std::wstring
-Strutil::utf8_to_utf16(string_view str)
+Strutil::utf8_to_utf16(string_view str) noexcept
 {
     std::wstring native;
 
@@ -624,7 +666,7 @@ Strutil::utf8_to_utf16(string_view str)
 
 
 std::string
-Strutil::utf16_to_utf8(const std::wstring& str)
+Strutil::utf16_to_utf8(const std::wstring& str) noexcept
 {
     std::string utf8;
 
@@ -640,7 +682,7 @@ Strutil::utf16_to_utf8(const std::wstring& str)
 
 
 char*
-Strutil::safe_strcpy(char* dst, string_view src, size_t size)
+Strutil::safe_strcpy(char* dst, string_view src, size_t size) noexcept
 {
     if (src.size()) {
         size_t end = std::min(size - 1, src.size());
@@ -658,16 +700,26 @@ Strutil::safe_strcpy(char* dst, string_view src, size_t size)
 
 
 void
-Strutil::skip_whitespace(string_view& str)
+Strutil::skip_whitespace(string_view& str) noexcept
 {
-    while (str.size() && isspace(str[0]))
+    while (str.size() && isspace(str.front()))
         str.remove_prefix(1);
 }
 
 
 
+void
+Strutil::remove_trailing_whitespace(string_view& str) noexcept
+{
+    while (str.size() && isspace(str.back()))
+        str.remove_suffix(1);
+}
+
+
+
 bool
-Strutil::parse_char(string_view& str, char c, bool skip_whitespace, bool eat)
+Strutil::parse_char(string_view& str, char c, bool skip_whitespace,
+                    bool eat) noexcept
 {
     string_view p = str;
     if (skip_whitespace)
@@ -685,7 +737,7 @@ Strutil::parse_char(string_view& str, char c, bool skip_whitespace, bool eat)
 
 
 bool
-Strutil::parse_until_char(string_view& str, char c, bool eat)
+Strutil::parse_until_char(string_view& str, char c, bool eat) noexcept
 {
     string_view p = str;
     while (p.size() && p[0] != c)
@@ -698,7 +750,7 @@ Strutil::parse_until_char(string_view& str, char c, bool eat)
 
 
 bool
-Strutil::parse_prefix(string_view& str, string_view prefix, bool eat)
+Strutil::parse_prefix(string_view& str, string_view prefix, bool eat) noexcept
 {
     string_view p = str;
     skip_whitespace(p);
@@ -714,7 +766,7 @@ Strutil::parse_prefix(string_view& str, string_view prefix, bool eat)
 
 
 bool
-Strutil::parse_int(string_view& str, int& val, bool eat)
+Strutil::parse_int(string_view& str, int& val, bool eat) noexcept
 {
     string_view p = str;
     skip_whitespace(p);
@@ -733,7 +785,7 @@ Strutil::parse_int(string_view& str, int& val, bool eat)
 
 
 bool
-Strutil::parse_float(string_view& str, float& val, bool eat)
+Strutil::parse_float(string_view& str, float& val, bool eat) noexcept
 {
     string_view p = str;
     skip_whitespace(p);
@@ -753,7 +805,7 @@ Strutil::parse_float(string_view& str, float& val, bool eat)
 
 bool
 Strutil::parse_string(string_view& str, string_view& val, bool eat,
-                      QuoteBehavior keep_quotes)
+                      QuoteBehavior keep_quotes) noexcept
 {
     string_view p = str;
     skip_whitespace(p);
@@ -762,13 +814,13 @@ Strutil::parse_string(string_view& str, string_view& val, bool eat,
     char lead_char    = p.front();
     bool quoted       = parse_char(p, '\"') || parse_char(p, '\'');
     const char *begin = p.begin(), *end = p.begin();
-    bool escaped = false;
+    bool escaped = false;  // was the prior character a backslash
     while (end != p.end()) {
         if (isspace(*end) && !quoted)
             break;  // not quoted and we hit whitespace: we're done
         if (quoted && *end == lead_char && !escaped)
-            break;  // closing quite -- we're done (beware embedded quote)
-        escaped = (p[0] == '\\');
+            break;  // closing quote -- we're done (beware embedded quote)
+        escaped = (end[0] == '\\') && (!escaped);
         ++end;
     }
     if (quoted && keep_quotes == KeepQuotes) {
@@ -790,7 +842,7 @@ Strutil::parse_string(string_view& str, string_view& val, bool eat,
 
 
 string_view
-Strutil::parse_word(string_view& str, bool eat)
+Strutil::parse_word(string_view& str, bool eat) noexcept
 {
     string_view p = str;
     skip_whitespace(p);
@@ -808,7 +860,7 @@ Strutil::parse_word(string_view& str, bool eat)
 
 
 string_view
-Strutil::parse_identifier(string_view& str, bool eat)
+Strutil::parse_identifier(string_view& str, bool eat) noexcept
 {
     string_view p = str;
     skip_whitespace(p);
@@ -829,7 +881,8 @@ Strutil::parse_identifier(string_view& str, bool eat)
 
 
 string_view
-Strutil::parse_identifier(string_view& str, string_view allowed, bool eat)
+Strutil::parse_identifier(string_view& str, string_view allowed,
+                          bool eat) noexcept
 {
     string_view p = str;
     skip_whitespace(p);
@@ -854,7 +907,8 @@ Strutil::parse_identifier(string_view& str, string_view allowed, bool eat)
 
 
 bool
-Strutil::parse_identifier_if(string_view& str, string_view id, bool eat)
+Strutil::parse_identifier_if(string_view& str, string_view id,
+                             bool eat) noexcept
 {
     string_view head = parse_identifier(str, false /* don't eat */);
     if (head == id) {
@@ -868,7 +922,7 @@ Strutil::parse_identifier_if(string_view& str, string_view id, bool eat)
 
 
 string_view
-Strutil::parse_until(string_view& str, string_view sep, bool eat)
+Strutil::parse_until(string_view& str, string_view sep, bool eat) noexcept
 {
     string_view p     = str;
     const char *begin = p.begin(), *end = p.begin();
@@ -883,8 +937,26 @@ Strutil::parse_until(string_view& str, string_view sep, bool eat)
 }
 
 
+
 string_view
-Strutil::parse_nested(string_view& str, bool eat)
+Strutil::parse_while(string_view& str, string_view set, bool eat) noexcept
+{
+    string_view p     = str;
+    const char *begin = p.begin(), *end = p.begin();
+    while (end != p.end() && set.find(*end) != string_view::npos)
+        ++end;
+    size_t wordlen = end - begin;
+    if (eat && wordlen) {
+        p.remove_prefix(wordlen);
+        str = p;
+    }
+    return string_view(begin, wordlen);
+}
+
+
+
+string_view
+Strutil::parse_nested(string_view& str, bool eat) noexcept
 {
     // Make sure we have a valid string and ascertain the characters that
     // nest and unnest.
@@ -927,10 +999,31 @@ Strutil::parse_nested(string_view& str, bool eat)
 
 
 
+std::string
+Strutil::excise_string_after_head(std::string& str, string_view head)
+{
+    std::string result;
+    string_view s(str);
+    size_t pattern_start = s.find(head);
+    if (pattern_start != string_view::npos) {
+        // Reposition s to be after the head
+        s.remove_prefix(pattern_start + head.size());
+        string_view m = Strutil::parse_until(s, " \t\r\n");
+        Strutil::skip_whitespace(s);
+        result = m;
+        str    = str.substr(0, pattern_start) + std::string(s);
+    }
+    return result;
+}
+
+
+
 /*
 Copyright for decode function:
 See http://bjoern.hoehrmann.de/utf-8/decoder/dfa/ for details.
 
+MIT license
+
 Copyright (c) 2008-2009 Bjoern Hoehrmann <bjoern@hoehrmann.de>
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -1004,6 +1097,10 @@ Strutil::utf8_to_unicode(string_view str, std::vector<uint32_t>& uvec)
 
 
 /* base64 code is based upon: http://www.adp-gmbh.ch/cpp/common/base64.html
+   https://github.com/ReneNyffenegger/cpp-base64
+
+   Zlib license
+
    Copyright (C) 2004-2008 René Nyffenegger
 
    This source code is provided 'as-is', without any express or implied
@@ -1146,22 +1243,17 @@ Strutil::stoi(string_view str, size_t* pos, int base)
 
 
 float
-Strutil::strtof(const char* nptr, char** endptr)
+Strutil::strtof(const char* nptr, char** endptr) noexcept
 {
     // Can use strtod_l on platforms that support it
-#if defined(__linux__) || defined(__APPLE__) || defined(__FreeBSD__)           \
-    || defined(__FreeBSD_kernel__) || defined(__GLIBC__)
-    // static initialization inside function is thread-safe by C++11 rules!
-    static locale_t c_loc = newlocale(LC_ALL_MASK, "C", nullptr);
-#    ifdef __APPLE__
+#ifdef __APPLE__
     // On OSX, strtod_l is for some reason drastically faster than strtof_l.
     return static_cast<float>(strtod_l(nptr, endptr, c_loc));
-#    else
+#elif defined(__linux__) || defined(__FreeBSD__)                               \
+    || defined(__FreeBSD_kernel__) || defined(__GLIBC__)
     return strtof_l(nptr, endptr, c_loc);
-#    endif
 #elif defined(_WIN32)
     // Windows has _strtod_l
-    static _locale_t c_loc = _create_locale(LC_ALL, "C");
     return static_cast<float>(_strtod_l(nptr, endptr, c_loc));
 #else
     // On platforms without strtof_l...
@@ -1170,7 +1262,7 @@ Strutil::strtof(const char* nptr, char** endptr)
         = std::use_facet<std::numpunct<char>>(native).decimal_point();
     // If the native locale uses decimal, just directly use strtof.
     if (nativepoint == '.')
-        return strtof(nptr, endptr);
+        return ::strtof(nptr, endptr);
     // Complex case -- CHEAT by making a copy of the string and replacing
     // the decimal, then use system strtof!
     std::string s(nptr);
@@ -1183,23 +1275,21 @@ Strutil::strtof(const char* nptr, char** endptr)
         return d;
     }
     // No decimal point at all -- use regular strtof
-    return strtof(s.c_str(), endptr);
+    return ::strtof(s.c_str(), endptr);
 #endif
 }
 
 
 double
-Strutil::strtod(const char* nptr, char** endptr)
+Strutil::strtod(const char* nptr, char** endptr) noexcept
 {
     // Can use strtod_l on platforms that support it
 #if defined(__linux__) || defined(__APPLE__) || defined(__FreeBSD__)           \
     || defined(__FreeBSD_kernel__) || defined(__GLIBC__)
     // static initialization inside function is thread-safe by C++11 rules!
-    static locale_t c_loc = newlocale(LC_ALL_MASK, "C", nullptr);
     return strtod_l(nptr, endptr, c_loc);
 #elif defined(_WIN32)
     // Windows has _strtod_l
-    static _locale_t c_loc = _create_locale(LC_ALL, "C");
     return _strtod_l(nptr, endptr, c_loc);
 #else
     // On platforms without strtod_l...
@@ -1208,7 +1298,7 @@ Strutil::strtod(const char* nptr, char** endptr)
         = std::use_facet<std::numpunct<char>>(native).decimal_point();
     // If the native locale uses decimal, just directly use strtod.
     if (nativepoint == '.')
-        return strtod(nptr, endptr);
+        return ::strtod(nptr, endptr);
     // Complex case -- CHEAT by making a copy of the string and replacing
     // the decimal, then use system strtod!
     std::string s(nptr);
@@ -1221,7 +1311,7 @@ Strutil::strtod(const char* nptr, char** endptr)
         return d;
     }
     // No decimal point at all -- use regular strtod
-    return strtod(s.c_str(), endptr);
+    return ::strtod(s.c_str(), endptr);
 #endif
 }
 
