@@ -1,32 +1,6 @@
-/*
-  Copyright 2009 Larry Gritz and the other authors and contributors.
-  All Rights Reserved.
-
-  Redistribution and use in source and binary forms, with or without
-  modification, are permitted provided that the following conditions are
-  met:
-  * Redistributions of source code must retain the above copyright
-    notice, this list of conditions and the following disclaimer.
-  * Redistributions in binary form must reproduce the above copyright
-    notice, this list of conditions and the following disclaimer in the
-    documentation and/or other materials provided with the distribution.
-  * Neither the name of the software's owners nor the names of its
-    contributors may be used to endorse or promote products derived from
-    this software without specific prior written permission.
-  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-
-  (This is the Modified BSD License)
-*/
+// Copyright 2008-present Contributors to the OpenImageIO project.
+// SPDX-License-Identifier: BSD-3-Clause
+// https://github.com/OpenImageIO/oiio/blob/master/LICENSE.md
 
 #ifndef PYOPENIMAGEIO_PY_OIIO_H
 #define PYOPENIMAGEIO_PY_OIIO_H
@@ -37,8 +11,13 @@
 #    pragma GCC diagnostic ignored "-Wregister"
 #endif
 
+// clang-format off
 // Must include Python.h first to avoid certain warnings
+#ifdef _POSIX_C_SOURCE
+#  error "You must include Python.h (and therefore py_oiio.h) BEFORE anything that defines _POSIX_C_SOURCE"
+#endif 
 #include <Python.h>
+// clang-format on
 
 #include <memory>
 
@@ -127,13 +106,13 @@ py_indexable_pod_to_stdvector(std::vector<T>& vals, const PYT& obj)
     for (size_t i = 0; i < length; ++i) {
         auto elem = obj[i];
         if (std::is_same<T, float>::value && py::isinstance<py::float_>(elem)) {
-            vals.emplace_back(elem.template cast<float>());
+            vals.emplace_back(T(elem.template cast<float>()));
         } else if ((std::is_same<T, float>::value || std::is_same<T, int>::value)
                    && py::isinstance<py::int_>(elem)) {
-            vals.emplace_back(elem.template cast<int>());
+            vals.emplace_back(T(elem.template cast<int>()));
         } else if (std::is_same<T, unsigned int>::value
                    && py::isinstance<py::int_>(elem)) {
-            vals.emplace_back(elem.template cast<unsigned int>());
+            vals.emplace_back(T(elem.template cast<unsigned int>()));
         } else {
             // FIXME? Other cases?
             vals.emplace_back(T(42));
@@ -483,6 +462,60 @@ make_numpy_array(TypeDesc format, void* data, int dims, size_t chans,
 }
 
 
+
+inline py::object
+ParamValue_getitem(const ParamValue& self, int n = 0)
+{
+    if (n < 0 || n >= self.nvalues()) {
+        throw std::out_of_range(
+            Strutil::sprintf("ParamValue index out of range %d", n));
+    }
+
+    TypeDesc t = self.type();
+
+#define ParamValue_convert_dispatch(TYPE)                                      \
+case TypeDesc::TYPE:                                                           \
+    return C_to_val_or_tuple((CType<TypeDesc::TYPE>::type*)self.data(), t)
+
+    switch (t.basetype) {
+        // ParamValue_convert_dispatch(UCHAR);
+        // ParamValue_convert_dispatch(CHAR);
+        ParamValue_convert_dispatch(USHORT);
+        ParamValue_convert_dispatch(SHORT);
+        ParamValue_convert_dispatch(UINT);
+        ParamValue_convert_dispatch(INT);
+        // ParamValue_convert_dispatch(ULONGLONG);
+        // ParamValue_convert_dispatch(LONGLONG);
+#ifdef _HALF_H_
+        ParamValue_convert_dispatch(HALF);
+#endif
+        ParamValue_convert_dispatch(FLOAT);
+        ParamValue_convert_dispatch(DOUBLE);
+    case TypeDesc::STRING:
+        return C_to_val_or_tuple((const char**)self.data(), t);
+    default: return py::none();
+    }
+
+#undef ParamValue_convert_dispatch
+}
+
+
+
+template<typename C>
+inline void
+delegate_setitem(C& self, const std::string& key, py::object obj)
+{
+    if (py::isinstance<py::float_>(obj))
+        self[key] = float(obj.template cast<py::float_>());
+    else if (py::isinstance<py::int_>(obj))
+        self[key] = int(obj.template cast<py::int_>());
+    else if (py::isinstance<py::str>(obj))
+        self[key] = std::string(obj.template cast<py::str>());
+    else
+        throw std::invalid_argument("Bad type for __setitem__");
+}
+
+
 }  // namespace PyOpenImageIO
 
 #endif  // PYOPENIMAGEIO_PY_OIIO_H
